#!/bin/ksh
#==========================================================================
#
# Script Name:  
#
#   ENS_EnsembleDriver.ksh
#
# Description:  
#
#   Top-level ensemble controller.  Called from cron or interactively.  
#
#   Starts job streams for all ensemble members including ensemble-wide 
#   post-processing.  
#    - Finds most recent complete set of input data (JGE+agrmet+sst) (AFWA 
#      only).  If there is a second command line argument, it is the target 
#      date we are going to use for GFS data.
#    - Checks stored flag files to see if an ensemble has already been run 
#      using this input data (AFWA only).  If the ensemble has already been 
#      run, this script writes a message to the log file and exits.  
#    - Creates a new directory called the "ensemble directory" into which 
#      ensemble results will be stored.  The name of the ensemble directory is 
#      derived from the first argument and from the and from the date and 
#      time of the recent complete set of input data (AFWA).  At both NCAR 
#      and AFWA, a second argument can be used to specify a retrospective 
#      data set instead.  (Note that the second argument is required in 
#      practice at NCAR since real-time data are not available.)  
#    - Calls ENS_EnsembleStart.ksh once for all ensemble members, passing in 
#      the ensemble directory, number of ensemble members, and valid dates of 
#      most recent GFS ensemble output data sets.  
#    - Prints results to a log file named ENS_EnsembleDriver.log in the 
#      ensemble directory.  
#
# Dependencies:  
#
#   Since this script is intended to be called from cron, it has no 
#   dependencies on environment variables.  
#
#   Scripts called by this script must reside in the same directory as 
#   this script.  
#
# Usage:
#  ENS_EnsembleDriver.ksh [-i] [-m] ensemble_top [retrospective_date]
#    OPTIONS:  
#      -i                  interactive:  wait until all jobs are done before 
#                          returning
#      -m                  do MSS writes instead of post (NCAR only)
#    REQUIRED ARGUMENTS:  
#      ensemble_top        full path to top-level ensemble directory
#      config_tag          Tag for config files and namelist directory,  can
#                          be, e.g. NCAR, DEMO, or ""
#      retrospective_date  YYYYMMDDHH for retrospective run
#
#   A single argument, ensemble_top, is required.  It must be the path to the 
#   top-level ensemble directory, which will be created if it does not 
#   already exist.  All ensemble output is stored beneath this top-level 
#   directory.  
#
#   A second argument, retrospective_date, is optionally accepted.  If this 
#   argument is not present, then this is assumed to be a production run and 
#   input data is searched for in locations specified in the configuration 
#   files.  If the argument is present, then this is assumed to be a 
#   retrospective run and the argument must be a date in the form 
#   YYYYMMDDHH .  
#
# Authors:  
#
#   Tom Henderson                          email:  hender@ucar.edu
#     NCAR, MMM Division                   voice:  303 497 1628
#     3450 Mitchell Lane                     fax:  303 497 8181
#     Boulder, Colorado 80301-2260 USA
#   Julie Schramm                          email:  schramm@ucar.edu
#     NCAR, MMM Division                   voice:  303 497 8191
#     3450 Mitchell Lane                     fax:  303 497 8181
#     Boulder, Colorado 80301-2260 USA
#   Dave Gill                              email:  gill@ucar.edu
#     NCAR, MMM Division                   voice:  303 497 8162
#     3450 Mitchell Lane                     fax:  303 497 8181
#     Boulder, Colorado 80301-2260 USA
#   Josh Hacker                            email:  hacker@ucar.edu
#     NCAR, RAL Division                   voice:  303 497 8188
#     3450 Mitchell Lane                     fax:  303 497 8181
#     Boulder, Colorado 80301-2260 USA
#

set -u

#==========================================================================
# name and location of this script
#-------------------------------------------------------------------------
self=$(/usr/bin/basename $0)
#-------------------------------------------------------------------------
# This method of determining scriptdir only works here because this file 
# is executed from the command line or cron and not from within LoadLeveler.  
#-------------------------------------------------------------------------
relscriptdir=$(/usr/bin/dirname $0)
if [[ $relscriptdir = /* ]] ; then
  scriptdir=$relscriptdir
else
  scriptdir="$( pwd )/$relscriptdir"
fi

#-------------------------------------------------------------------------
# parse command line
#-------------------------------------------------------------------------
CommandLineError=FALSE

# parse command-line options
InteractiveRun=FALSE  # wait until all jobs are done before returning if TRUE
MassStore=FALSE       # do MSS writes instead of post if TRUE (NCAR only)
while getopts :im arg
do
  case $arg in
    i)
      InteractiveRun=TRUE
      ;;
    m)
      MassStore=TRUE
      ;;
    \?)
      CommandLineError=TRUE
      ;;
  esac
done
#-------------------------------------------------------------------------
# get rid of parsed options
#-------------------------------------------------------------------------
shift $(($OPTIND - 1))

#TODO:  Command-line parsing does not correctly parse option flags that follow 
#TODO:  arguments.  (i.e. "ENS_EnsembleDriver.ksh mydir -i").  Either fix 
#TODO:  this or detect it and exit.  

#-------------------------------------------------------------------------
# Are we running real-time (one arg) or are we running a retrospective case 
# at a requested time (two args)?  
#-------------------------------------------------------------------------
if (( $# == 3 )) ; then
  ensemble_top=$1
  config_tag=$2
  ForceDate=$3          # must be YYYYMMDDHH
  RetrospectiveRun=TRUE
#elif (( $# == 2 )) ; then
#  ensemble_top=$1
#  config_tag=$2
#  RetrospectiveRun=FALSE
else
  CommandLineError=TRUE
fi

#-------------------------------------------------------------------------
# explicitly load all shared functions
#-------------------------------------------------------------------------
. ${scriptdir}/ENS_SharedFunctions.ksh
#-------------------------------------------------------------------------
# explicitly set path
#-------------------------------------------------------------------------
SetPath
GetUser  # sets ENS_User
#-------------------------------------------------------------------------
# Figure out what site we are at...  
#-------------------------------------------------------------------------
GetSite  # sets ENS_Site
#-------------------------------------------------------------------------
# email address for notification of job completion or errors (NCAR only)
#-------------------------------------------------------------------------
email_target="${ENS_User}@ucar.edu"

#-------------------------------------------------------------------------
# defer command-line error-exit until after shared functions are loaded
#-------------------------------------------------------------------------
if [[ $CommandLineError = "TRUE" ]] ; then
  Usage
  ErrorExit "Command-line ERROR:  ENS_EnsembleDriver.ksh could not parse arguments" ${self}
fi

#-------------------------------------------------------------------------
# Log file date is current date.  
#-------------------------------------------------------------------------
datecmd="date -u"    # use GMT
logdatestr="$(${datecmd} +%Y%m%d_%H%M%Z)"
ensemble_log_dir=${ensemble_top}/logs/${logdatestr}

#-------------------------------------------------------------------------
# start log file
# $$$here...  must figure out a way to send email for this error...  
#-------------------------------------------------------------------------
mkdir -p ${ensemble_log_dir} || \
  ErrorExit "could not create log directory ${ensemble_log_dir}" ${self}

logfile="${ensemble_log_dir}/ENS_EnsembleDriver.log"
#-------------------------------------------------------------------------
# $$$here...  must figure out a way to send email for this error...  
#-------------------------------------------------------------------------
print "BEGIN ENS ENSEMBLE:  $(${datecmd} +%Y%m%d) $(${datecmd} +%H:%M:%S) $(${datecmd} +%Z)\n" > \
  $logfile || \
  ErrorExit "could not create log file ${logfile}" ${self}

#-------------------------------------------------------------------------
# Once $logfile has been defined, configuration files can be read, and 
# email can be sent for error messages...  

# parse ensemble-wide config file
# GetConfig sets variables named ENS_CONFIG_*
#-------------------------------------------------------------------------
GetMyConfig 0 $scriptdir $logfile $email_target $config_tag

# turn on for debugging
ENS_CONFIG_DEBUG="TRUE"
if [[ $ENS_CONFIG_DEBUG = "TRUE" ]] ; then
  set -x
fi

#-------------------------------------------------------------------------
# Create new ensemble directory based either upon valid date of most recent 
# JGE data set or the requested date (AFWA and NCAR).  
#-------------------------------------------------------------------------
  if [[ $RetrospectiveRun = "FALSE" ]] ; then
    #---------------------------------------------------------------------
    # sets ENS_JGEValidDate, ENS_agrmetValidDate, and ENS_sstValidDate
    #---------------------------------------------------------------------
    GetMostRecentValidDate $logfile $email_target
  else
    #---------------------------------------------------------------------
    # sets ENS_JGEValidDate, ENS_agrmetValidDate, and ENS_sstValidDate
    #---------------------------------------------------------------------
    ForceValidDate $logfile $email_target $ForceDate
  fi
  ensemble_dir=${ensemble_top}/${ENS_JGEValidDate}

  #-----------------------------------------------------------------------
  # check for flag file
  #-----------------------------------------------------------------------
  flag_path=${ensemble_top}/flags
  done_file=${flag_path}/DONE.${ENS_JGEValidDate}
  running_file=${flag_path}/RUNNING.${ENS_JGEValidDate}
  if [[ -d ${flag_path} ]] ; then
    if [[ -f ${running_file} ]] ; then
      AppendToLogFile "${self}:  ENSEMBLE ALREADY RUNNING for ${ENS_JGEValidDate}, exitting" \
        $self $scriptdir $logfile $email_target
      exit 0
    fi
  else
    #---------------------------------------------------------------------
    # create directory for "done" flags if it does not already exist
    #---------------------------------------------------------------------
    mkdir -p ${flag_path}
    if (( $? != 0 )) ; then
      ErrorEmailExit "could not create flag directory ${flag_path}" \
        ${self} $logfile $email_target ${ENS_JGEValidDate}
    fi
  fi
  print "ENSEMBLE RUNNING FOR ${ENS_JGEValidDate}" > $running_file

#-------------------------------------------------------------------------
# create ensemble_dir
#-------------------------------------------------------------------------
if [[ -d ${ensemble_dir} ]] ; then
  if [[ $ENS_Site = "AFWA" ]] ; then
    #---------------------------------------------------------------------
    # directory must not already exist (AFWA only)
    #---------------------------------------------------------------------
    ErrorEmailExit "ensemble directory ${ensemble_dir} already exists" \
      ${self} $logfile $email_target ${ENS_JGEValidDate}
  else
    #---------------------------------------------------------------------
    # move directory if it already exists (NCAR only)
    #---------------------------------------------------------------------
    mv ${ensemble_dir} ${ensemble_dir}.${$}.OLD
    if (( $? != 0 )) ; then
      ErrorEmailExit "could move old ensemble directory ${ensemble_dir}" \
        ${self} $logfile $email_target ${ENS_JGEValidDate}
    fi
  fi
fi
mkdir -p ${ensemble_dir}
if (( $? != 0 )) ; then
  ErrorEmailExit "could not create ensemble directory ${ensemble_dir}" \
    ${self} $logfile $email_target ${ENS_JGEValidDate}
fi
loginfofile=${ensemble_dir}/README.logfiles
print "Log files for this run are in ${ensemble_log_dir}" > ${loginfofile}
if (( $? != 0 )) ; then
  ErrorEmailExit "could not write log file info to ${loginfofile}" \
    ${self} $logfile $email_target ${ENS_JGEValidDate}
fi

#-------------------------------------------------------------------------
# start writing all sorts of useful stuff to the log file
#-------------------------------------------------------------------------
AppendToLogFile "${self}:  Working in ensemble directory ${ensemble_dir}" \
  $self $scriptdir $logfile $email_target
AppendToLogFile "${self}:  Script directory is ${scriptdir}" \
  $self $scriptdir $logfile $email_target

#-------------------------------------------------------------------------
# Log which ensemble members are to be run and the total number
#-------------------------------------------------------------------------
typeset -u ENS_CONFIG_RunMembers
if [[ $ENS_CONFIG_RunMembers = ALL ]] ; then
   #-----------------------------------------------------------------------
   #  Run all members 1-ENS_CONFIG_NumMembers
   #-----------------------------------------------------------------------
   inputMemberString="1-${ENS_CONFIG_NumMembers}"
else
   #-----------------------------------------------------------------------
   #  Run selected members
   #-----------------------------------------------------------------------
   inputMemberString=$ENS_CONFIG_RunMembers
fi

AppendToLogFile "${self}:  Input string of ensemble members is ${inputMemberString}" \
  $self $scriptdir $logfile $email_target

memberString=`$scriptdir/getMemberIds.pl -s ${inputMemberString}`
set -A memberId $memberString

AppendToLogFile "${self}:  Array of ensemble members is ${memberId[*]}" \
  $self $scriptdir $logfile $email_target

maxensemble=${#memberId[*]}

AppendToLogFile "${self}:  Number of ensemble members is ${maxensemble}" \
  $self $scriptdir $logfile $email_target

#-------------------------------------------------------------------------
# Do some error checking on ensemble members, # elements must be > 0
#-------------------------------------------------------------------------

if (($maxensemble <= 0)) ; then
   ErrorEmailExit "ENS_EnsembleDriver.ksh failed: maxensemble < or = 0" \
      $self $logfile $email_target
fi

#-------------------------------------------------------------------------
# All elements must be <= ENS_CONFIG_NumMembers
#-------------------------------------------------------------------------
for aMember in ${memberId[*]} ; do
   if (( $aMember > $ENS_CONFIG_NumMembers )) ; then
      ErrorEmailExit "ENS_EnsembleDriver.ksh failed: $aMember > ENS_CONFIG_NumMembers" \
         $self $logfile $email_target
   fi
done

# command-line options
if [[ $InteractiveRun = "TRUE" ]] ; then
  if [[ $ENS_Site = "AFWA" ]] ; then
    ErrorEmailExit "Cannot run interactively at AFWA" \
      ${self} $logfile $email_target ${ENS_JGEValidDate}
  else
    AppendToLogFile "${self}:  This is an interactive run." \
      $self $scriptdir $logfile $email_target
  fi
fi
if [[ $MassStore = "TRUE" ]] ; then
  if [[ $ENS_Site = "AFWA" ]] ; then
    ErrorEmailExit "Cannot write to MSS at AFWA" \
      ${self} $logfile $email_target ${ENS_JGEValidDate}
  else
    AppendToLogFile "${self}:  Output will be written to the NCAR MSS" \
      $self $scriptdir $logfile $email_target
  fi
fi

#-------------------------------------------------------------------------
# start entire ensemble
#-------------------------------------------------------------------------
AppendToLogFile "${self}:  Starting ${maxensemble} ensemble members" \
  $self $scriptdir $logfile $email_target
jobid=$( ${scriptdir}/ENS_EnsembleStart.ksh $scriptdir $maxensemble ${memberId[*]} \
  $ensemble_dir $ENS_JGEValidDate $ENS_agrmetValidDate $ENS_sstValidDate \
  $RetrospectiveRun $InteractiveRun $MassStore $logfile $email_target \
  $config_tag)
if (( $? != 0 )) ; then
  ErrorEmailExit "ENS_EnsembleStart.ksh failed" \
    $self $logfile $email_target ${ENS_JGEValidDate}
fi

#-------------------------------------------------------------------------
# Figure out what batch queueing system we are using
#-------------------------------------------------------------------------
GetBatch  # sets ENS_Batch
if [[ $ENS_Batch = "LSF" ]] ; then
  AppendToLogFile "${self}:  Submitted LSF group == \"${jobid}\"" \
    $self $scriptdir $logfile $email_target
  print
  print "Submitted LSF group == \"${jobid}\""
elif [[ $ENS_Batch = "LOADLEVELER" ]] ; then
  AppendToLogFile "${self}:  Submitted LoadLeveler jobid == <${jobid}>" \
    $self $scriptdir $logfile $email_target
  print
  print "Submitted LoadLeveler jobid == <${jobid}>"
else
  AppendToLogFile "${self}:  Submitted serial jobid == <${jobid}>" \
    $self $scriptdir $logfile $email_target
  print
  print "Submitted serial jobid == <${jobid}>"
fi

AppendToLogFile "END ${self}:  $(${datecmd} +%Y%m%d) $(${datecmd} +%H:%M:%S) $(${datecmd} +%Z)" \
  $self $scriptdir $logfile $email_target 

print
print "See log file ${logfile} and ensemble directory ${ensemble_dir}"
print "for results once job ${jobid} completes."
print

# TBH:  Old bit of code used for testing.  
# AppendToLogFile "${self}:  Waiting for all ensemble members" \
#   $self $scriptdir $logfile $email_target
# sleep 3   # wait a bit for job to queue
# ${scriptdir}/ENS_EnsembleWait.ksh $scriptdir $jobid \
#   $ensemble_dir $logfile $email_target || \
#   ErrorEmailExit "ENS_EnsembleWait.ksh failed" \
#     $self $logfile $email_target ${ENS_JGEValidDate}

exit 0

