#!/bin/ksh
#
# Script Name:
#
#   ENS_EnsemblePost.ksh
#
# Usage:  
#
#   ENS_EnsemblePost.ksh $scriptdir $maxensemble $ensemble_dir $ENS_ValidDate \
#                        $successflag $write_netcdf_to_mss $logfile $email_target
#
#     $scriptdir           absolute path to these scripts
#     $maxensemble         number of ensembles
#     $ensemble_dir        top-level working directory
#     $ENS_ValidDate       valid start date of forecast
#     $successflag         "SUCCESS" if all previous job steps have retured 
#                          exit status zero, "FAILURE" otherwise.  
#     $write_netcdf_to_mss write log files files to MSS if TRUE (NCAR only)
#     $logfile             full path to logfile
#     $email_target        where to send email if something fails
#     $config_tag          posterior tag on configs and namelists, e.g. DEMO
#
# Description:  
#
#   Post-processing for entire ensemble.  
#
#   $$$ This is where Steve Klotz's application could be called...  
#
#   This script then creates a "flag" file to indicate completion to other 
#   applications.  
#   NOTE:  This script should NOT return until the flag file has been handled.  
#
#   This script mails log file to recipients encoded herein (NCAR only).  
#
#   If $write_netcdf_to_mss is "TRUE", log files will be written to the 
#   MSS (NCAR only).  
#
# Dependencies:  
#
#   Since this script is intended to be called from a script called from cron, 
#   it has no dependencies on environment variables.  
#
#   Scripts called by this script must reside in the same directory as 
#   this script.  
#
# Author:  
#
#   Tom Henderson                          email:  hender@ucar.edu
#   NCAR, MMM Division                     voice:  303 497 1628
#   3450 Mitchell Lane                       fax:  303 497 8181
#   Boulder, Colorado 80301-2260 USA
#

set -u

# name of this script
self=$(/usr/bin/basename $0)

# load arguments
scriptdir=$1; shift
maxensemble=$1; shift
ensemble_dir=$1; shift
ENS_ValidDate=$1; shift
successflag=$1; shift
write_netcdf_to_mss=$1; shift
logfile=$1; shift
email_target=$1; shift
config_tag=$1; shift

# explicitly load all shared functions
. ${scriptdir}/ENS_SharedFunctions.ksh
# explicitly set path
SetPath

# parse ensemble-wide config file
# GetConfig sets variables named ENS_CONFIG_*
GetMyConfig 0 $scriptdir $logfile $email_target $config_tag

# turn on for debugging
if [[ $ENS_CONFIG_DEBUG = "TRUE" ]] ; then
  set -x
fi

AppendToLogFile "${self}:  WRF ENSEMBLE POST BEGIN at $(date -u)" \
  $self $scriptdir $logfile $email_target

AppendToLogFile "${self}:  ENS start date = ${ENS_ValidDate}" \
  $self $scriptdir $logfile $email_target

log_path=$(/usr/bin/dirname ${logfile})
if [[ $successflag = "SUCCESS" ]] ; then
  # check for failure messages in log files
  # this is a bit of a hack but return status are not so easily passed back
  # from batch jobs
  grep -i fail ${log_path}/ENS_EnsembleDriver.log* > /dev/null 2>&1
  if (( $? == 0 )) ; then
    # something broke
    result="FAILURE"
  else
    # successful completion
    result="SUCCESS"
  fi
else
  result="FAILURE"
fi

if [[ $result = "SUCCESS" ]] ; then
  AppendToLogFile "\nENS SUCCESS, begin ensemble-wide post-processing\n" \
    $self $scriptdir $logfile $email_target
  if [[ $write_netcdf_to_mss = "FALSE" ]] ; then
    AppendToLogFile "${self}:  I do not do anything yet..." \
      $self $scriptdir $logfile $email_target
  fi
# $$$ add Steve Klotz's ensemble-wide post-processing here...  
else
  AppendToLogFile "\nENS FAILURE\nOne or more component(s) failed, see log files in this directory\n" \
    $self $scriptdir $logfile $email_target
fi


# $$$ TODO:  Improve error handling here and below...

# create flag file
# $$$ TODO:  Change to pass $done_file, $running_file, etc. in via interface...
flag_path=${ensemble_dir}/../flags
done_file=${flag_path}/DONE.${ENS_ValidDate}
running_file=${flag_path}/RUNNING.${ENS_ValidDate}
if [[ $result = "SUCCESS" ]] ; then
  print "ENSEMBLE DONE FOR ${ENS_ValidDate}" > $done_file
else
  print "ENSEMBLE DONE FOR ${ENS_ValidDate}, but some member(s) FAILED, see log files in ${log_path}" > $done_file
fi
\rm -f $running_file

# change permissions so everyone can read everything
for fixpath in ${ensemble_dir} ${log_path} ${flag_path} ; do
  find $fixpath -type d -exec chmod ug+rwx {} \;
  find $fixpath -type f -exec chmod ug+rw {} \;
done

# OK to use ErrorEmailExit below this point...  

# Archive log files on MSS (NCAR only)
if [[ $write_netcdf_to_mss = "TRUE" ]] ; then
  AppendToLogFile "${self}:  WRF ENSEMBLE POST Begin MSS transfer of log files at $(date -u)" \
    $self $scriptdir $logfile $email_target
  cd ${log_path} || \
    ErrorEmailExit "ENSEMBLE_POST_FAIL:  Could not cd to ${log_path}" \
      $self $logfile $email_target $ENS_ValidDate
  AppendToLogFile "${self}:  Moving log files to MSS" \
    $self $scriptdir $logfile $email_target
  # NOTE:  Changes to logfiles made below this point will not be archived on 
  # NOTE:  the MSS.  
  tarfile="logs.tar"
# TODO:  warning -- duplication!
  tarcmd="tar cf ${tarfile} ENS_EnsembleDriver.log* config_*"
# TODO:  put this in a function & remove duplication with ENS_MemberPost.ksh
  AppendToLogFile "$tarcmd" $self $scriptdir $logfile $email_target
  tar cf ${tarfile} ENS_EnsembleDriver.log* config_* || \
    ErrorEmailExit "ENSEMBLE_POST_FAIL:  error tarring log files" \
      $self $logfile $email_target $ENS_ValidDate
# TBH:  Removed gzip for now since it can be slow for long runs.
  msscmd="msrcp -period 3650 -project $ENS_CONFIG_Account -srcdelete ${tarfile} ${ENS_CONFIG_MSSPath}/${ENS_ValidDate}/${tarfile}"
  AppendToLogFile "$msscmd" $self $scriptdir $logfile $email_target
  msrcp -period 3650 -project $ENS_CONFIG_Account -srcdelete ${tarfile} ${ENS_CONFIG_MSSPath}/${ENS_ValidDate}/${tarfile} || \
    ErrorEmailExit "ENSEMBLE_POST_FAIL:  Could not transfer ${tarfile} to MSS" \
      $self $logfile $email_target $ENS_ValidDate
  AppendToLogFile "${self}:  WRF ENSEMBLE POST End MSS transfer of log files at $(date -u)" \
    $self $scriptdir $logfile $email_target
fi 

# Get site and exit if not AFWA or NCAR
GetSite
if [[ $ENS_CONFIG_PostLevel > 0 ]] ; then
  AppendToLogFile "${self}:  Calling postprocessor level $ENS_CONFIG_PostLevel at $(date -u)" \
  $self $scriptdir $logfile $email_target
  ${scriptdir}/ENS_WRF_Post_L${ENS_CONFIG_PostLevel}.ksh $scriptdir \
               $maxensemble $ensemble_dir $ENS_ValidDate $logfile \
               $email_target $config_tag
  return
fi


AppendToLogFile "${self}:  WRF ENSEMBLE POST END at $(date -u)" \
  $self $scriptdir $logfile $email_target

datecmd="date -u"    # use GMT
AppendToLogFile "END ENS ENSEMBLE:  $(${datecmd} +%Y%m%d) $(${datecmd} +%H:%M:%S) $(${datecmd} +%Z)" \
  $self $scriptdir $logfile $email_target

# Mail log file indicating successful completion
if [[ ${ENS_CONFIG_EmailNotify} = "yes" ]] ; then
  mail -s "ENS ${result} ${script_name}:  top-level log file ${logfile} follows" $email_target < ${logfile}
fi

exit 0

