#!/bin/ksh
#==========================================================================
#
# Script Name:
#
#   ENS_EnsembleStart.ksh
#
# Usage:  
#
#   ENS_EnsembleStart.ksh $scriptdir $maxensemble ${memberId[*]} \
#                         $ensemble_dir $ENS_JGEValidDate        \
#                         $ENS_agrmetValidDate $ENS_sstValidDate \
#                         $RetrospectiveRun $InteractiveRun $MassStore \
#                         $logfile $email_target
#
#     $scriptdir           absolute path to these scripts
#     $maxensemble         number of ensemble members
#     ${memberId[*]}       array of ensemble members to run
#     $ensemble_dir        top-level working directory
#     $ENS_JGEValidDate    valid date of most recent complete data set
#     $ENS_agrmetValidDate valid date of corresponding agrment data set
#     $ENS_sstValidDate    valid date of corresponding sst data set
#     $RetrospectiveRun    TRUE if this is a retrospective run, FALSE for a 
#                          standard production run.  
#     $InteractiveRun      wait until all jobs are done before returning if 
#                          TRUE
#     $MassStore           do MSS writes instead of post if TRUE (NCAR only)
#     $logfile             full path to logfile
#     $email_target        where to send email if something fails
#     $config_tag          where to find configs and namelists
#
# Description:  
#
#   Ensemble-wide Job Command File (JCF) creation and job submission.  
#
#   Creates directories for each member, creates a multi-step LoadLeveler or 
#   LSF Job Command File (JCF) script to call the following scripts in order 
#   for each ensemble member:  
#     - ENS_WRF_WPS.ksh:  runs WPS, checks for successful completion, and 
#       creates namelists for real.exe and wrf.exe.  
#     - ENS_WRF_Real_WPS.ksh:  Runs WRF real.exe and
#       checks for successful completion.  
#     - ENS_WRF_Model.ksh:  Runs WRF wrf.exe and
#       checks for successful completion.  
#     - ENS_MemberPost.ksh:  "Member Post-Process" performs any
#       per-member post-processing and/or file movement required by the
#       "Ensemble Post-Process" script.
#   This script then appends a call to one instance of ENS_EnsemblePost.ksh 
#   to the JCF to process all ensemble results.
#   This script then submits the JCF it created to LoadLeveler or LSF, and 
#   returns the Job ID of the submitted job.  
#
#   Note that for LSF, the JCF is a wrapper that submits a whole bunch of 
#   LSF batch scripts, each of which handles one job step.  Yucko.  
#
#   This script also executes the IBM-specific "lslpp" command to record 
#   the state of the installed softwared stack as an aid in diagnosing 
#   production problems.  
#
# Output:  
#
#   For LoadLeveler, this script echoes the Job ID returned by llsubmit so the 
#   caller can check for job completion (if desired).  
#   For LSF, this script echoes the group ID used in JCFs so the caller can 
#   manage the whole group (if desired).  
#
# Dependencies:  
#
#   Since this script is intended to be called from a script called from cron, 
#   it has no dependencies on environment variables.  
#
#   Scripts called by this script must reside in the same directory as 
#   this script.  
#
# Authors:  
#
#   Tom Henderson                          email:  hender@ucar.edu
#   NCAR, MMM Division                     voice:  303 497 1628
#   3450 Mitchell Lane                       fax:  303 497 8181
#   Boulder, Colorado 80301-2260 USA
#
#   Josh Hacker                            email:  hacker@ucar.edu
#   NCAR RAL Division                      voice:  303 497 8188
#   3450 Mitchell Lane
#   Boulder, Colorado 80301-2260 USA
#
# Modifications by Josh Hacker from WRFSI-->WPS
#
#==========================================================================

set -u

# name of this script
#----------------------------------------------------------------------
self=$(/usr/bin/basename $0)

#----------------------------------------------------------------------
# load arguments
#----------------------------------------------------------------------
scriptdir=$1; shift
maxensemble=$1; shift

set -A memberId 
icount=0
until (( icount >= maxensemble )) ; do
  memberId[$icount]=$1; shift
  (( icount += 1 ))
done

ensemble_dir=$1; shift
ENS_JGEValidDate=$1; shift
ENS_agrmetValidDate=$1; shift
ENS_sstValidDate=$1; shift
RetrospectiveRun=$1; shift
InteractiveRun=$1; shift
MassStore=$1; shift
logfile=$1; shift
email_target=$1; shift
config_tag=$1; shift

#----------------------------------------------------------------------
# explicitly load all shared functions
#----------------------------------------------------------------------
. ${scriptdir}/ENS_SharedFunctions.ksh
#----------------------------------------------------------------------
# explicitly set path
#----------------------------------------------------------------------
SetPath

AppendToLogFile "${self}:  memberId is ${memberId[*]}" \
    $self $scriptdir $logfile $email_target
#----------------------------------------------------------------------
# parse ensemble-wide config file
# GetConfig sets variables named ENS_CONFIG_*
# these are needed by ErrorEmailExit
#----------------------------------------------------------------------
GetMyConfig 0 $scriptdir $logfile $email_target $config_tag

# turn on for debugging
if [[ $ENS_CONFIG_DEBUG = "TRUE" ]] ; then
  set -x
fi

#----------------------------------------------------------------------
# Figure out what site we are at...
#----------------------------------------------------------------------
GetSite  # sets ENS_Site
#----------------------------------------------------------------------
# Figure out what batch queueing system we are using and set JCF name...
#----------------------------------------------------------------------
GetBatch  # sets ENS_Batch
if [[ $ENS_Batch = "LSF" ]] ; then
  jobscript="${ensemble_dir}/WRF_ENS.lsf"
elif [[ $ENS_Batch = "LOADLEVELER" ]] ; then
  jobscript="${ensemble_dir}/WRF_ENS.ll"
else
  jobscript="${ensemble_dir}/WRF_ENS.job"
fi

#----------------------------------------------------------------------
# do our best to record the state of the software stack
# NOTE:  THIS BIT IS IBM-SPECIFIC
#----------------------------------------------------------------------
if [[ $ENS_Batch = "LSF" ]] ; then
lslppout="${ensemble_dir}/lslpp.out"
if [[ -f $lslppout ]] ; then
  ErrorEmailExit "lslpp output file ${lslppout} already exists" \
    $self $logfile $email_target $ENS_JGEValidDate
fi
touch $lslppout || \
  ErrorEmailExit "could not create lslpp output file ${lslppout}" \
    $self $logfile $email_target $ENS_JGEValidDate
print "BEGIN OUTPUT FROM \"xlf -qversion\" at $(date)" >> $lslppout
xlf -qversion >> $lslppout
print "END OUTPUT FROM \"xlf -qversion\" at $(date)" >> $lslppout
print "BEGIN OUTPUT FROM \"lslpp -ha\" at $(date)" >> $lslppout
lslpp -ha >> $lslppout
print "END OUTPUT FROM \"lslpp -ha\" at $(date)" >> $lslppout
fi
#----------------------------------------------------------------------
# NOTE:  END OF IBM-SPECIFIC BIT
#----------------------------------------------------------------------

#----------------------------------------------------------------------
# define name of LoadLeveler/LSF Job Command File script
#----------------------------------------------------------------------
if [[ -f $jobscript ]] ; then
  ErrorEmailExit "Job Command File ${jobscript} already exists" \
    $self $logfile $email_target $ENS_JGEValidDate
fi
touch $jobscript || \
  ErrorEmailExit "could not create Job Command File ${jobscript}" \
    $self $logfile $email_target $ENS_JGEValidDate

#----------------------------------------------------------------------
# Create run directories for each ensemble member
#----------------------------------------------------------------------
for thisensemble in ${memberId[*]} ; do
  #--------------------------------------------------------------------
  # parse config file for $thisensemble
  # GetMyConfig sets variables named ENS_CONFIG_*
  #--------------------------------------------------------------------
  GetMyConfig $thisensemble $scriptdir $logfile $email_target $config_tag
  runpath="${ensemble_dir}/${ENS_CONFIG_RunPath}"
  AppendToLogFile "${self}:  Creating run directory ${runpath} for \
                   ensemble member ${thisensemble} ..."        \
                   $self $scriptdir $logfile $email_target
  if [[ -d $runpath ]] ; then
    ErrorEmailExit "run directory for ensemble member            \
                    ${ENS_CONFIG_MemberName} already exists" \
                    $self $logfile $email_target $ENS_JGEValidDate
  fi
  mkdir -p $runpath || ErrorEmailExit "could not create run directory    \
                       for ensemble member ${ENS_CONFIG_MemberName}" \
                       $self $logfile $email_target $ENS_JGEValidDate
done

#----------------------------------------------------------------------
# Create LoadLeveler/LSF JCF or just a run script
#----------------------------------------------------------------------
AppendToLogFile "${self}:  Creating Job Command File ${jobscript} ..." \
  $self $scriptdir $logfile $email_target

if [[ $ENS_Batch == "LOADLEVELER" ||  $ENS_Batch == "LSF" ]] ; then

#----------------------------------------------------------------------
# create all sections of LoadLeveler/LSF JCF 
#----------------------------------------------------------------------
  set -A logMsgInfo  WRF_WPS  WRF_Real_WPS  WRF_Model  MemberPost
  set -A ENSJobType "WRFWPS" "WRFREAL_WPS" "WRFMODEL" "WRFPOST"
  set -A QueueName  $ENS_CONFIG_WPSQueueName $ENS_CONFIG_RealQueueName \
                  $ENS_CONFIG_WRFQueueName $ENS_CONFIG_PostQueueName
  set -A NumNodes   $ENS_CONFIG_WPSNumNodes  $ENS_CONFIG_RealNumNodes  \
                  $ENS_CONFIG_WRFNumNodes  $ENS_CONFIG_PostNumNodes
  set -A WallTime   $ENS_CONFIG_WPSWallTime  $ENS_CONFIG_RealWallTime  \
                  $ENS_CONFIG_WRFWallTime  $ENS_CONFIG_PostWallTime

  jobtype=parallel
else
#----------------------------------------------------------------------
# create all sections of a generic run script arguments
#----------------------------------------------------------------------
  AppendToLogFile "${self}: Not using a queuing system ..." \
  $self $scriptdir $logfile $email_target
  set -A logMsgInfo  WRF_WPS  WRF_Real_WPS  WRF_Model  MemberPost
  set -A ENSJobType "WRFWPS" "WRFREAL_WPS" "WRFMODEL" "WRFPOST"
  set -A QueueName  NONE NONE NONE NONE
  set -A NumNodes   NONE  NONE NONE  NONE
  set -A WallTime   NONE  NONE NONE  NONE
  ENS_CONFIG_NodeUsage=NONE
  ENS_CONFIG_TasksPerNode=NONE
  ENS_CONFIG_Account=NONE
  ENS_CONFIG_EnsPostWallTime=NONE
  ENS_CONFIG_PostQueueName=NONE
  jobtype=serial
fi

#----------------------------------------------------------------------
# Only need "bsub -K" for the last few jobs submitted.  
#----------------------------------------------------------------------
enable_interactive="FALSE"
step_num=0
jobCount=0

for logMsg in ${logMsgInfo[*]}; do

#----------------------------------------------------------------------
# Set any additional parameters needed by AddJobStep
#----------------------------------------------------------------------
   if [[ ${ENSJobType[$jobCount]} = "WRFWPS" ]] ; then
      set -A allotherargs $ENS_JGEValidDate $ENS_agrmetValidDate \
                          $ENS_sstValidDate $RetrospectiveRun
   elif [[ ${ENSJobType[$jobCount]} = "WRFREAL_WPS" ]] ; then
      set -A allotherargs $ENS_JGEValidDate 
   elif [[ ${ENSJobType[$jobCount]} = "WRFMODEL" ]] ; then
      set -A allotherargs $ENS_JGEValidDate 
   elif [[ ${ENSJobType[$jobCount]} = "WRFPOST" ]] ; then
      set -A allotherargs $ENS_JGEValidDate $MassStore
   else
      set -A allotherargs
   fi

   for thisensemble in ${memberId[*]} ; do

      GetMyConfig $thisensemble $scriptdir $logfile $email_target $config_tag

      logMessage="${self}:  Creating JCF section for ${logMsg} \
        for ensemble member ${thisensemble} ..."
      AppendToLogFile "$logMessage" $self $scriptdir $logfile $email_target

      logMessage="${self}:  Log file for ENS_${logMsg}.ksh \
        will be ${logfile}.ENS_${logMsg}.${thisensemble} ..." 
      AppendToLogFile "$logMessage" $self $scriptdir $logfile $email_target

      AddJobStep $jobscript $ENS_Batch $scriptdir $step_num ${QueueName[$jobCount]} \
                  $jobtype ${NumNodes[$jobCount]} \
                  ${scriptdir}/ENS_${logMsg}.ksh ${ENSJobType[$jobCount]} \
                  ${WallTime[$jobCount]} $thisensemble $maxensemble ${memberId[*]} \
                  $ensemble_dir $enable_interactive \
                  $logfile.ENS_${logMsg}.${thisensemble} $email_target \
                  $config_tag ${allotherargs[*]}

      (( step_num += 1 ))
   done
   (( jobCount += 1 ))
done

#----------------------------------------------------------------------
# create section of LoadLeveler/LSF JCF for ensemble-wide post-processing
# and to clean up in case ensemble-wide post-processing is not executed
# due to failure of a previous job step
# TODO:  Note that LSF depends on ENSFAIL being the last job added to the 
# TODO:  job scripts.  Remove this dependence in AddJobStep if needed...  
#----------------------------------------------------------------------
set -A ENSJobType "ENSPOST" "ENSFAIL"
set -A statusFlag "SUCCESS" "FAILURE"
set -A logMsgInfo  "ensemble-wide post-processing" "error handling"
ensCount=0

for jobType in ${ENSJobType[*]}; do
   AppendToLogFile "${self}:  Creating JCF section for ${logMsgInfo[$ensCount]} ..." \
                    $self $scriptdir $logfile $email_target
   AppendToLogFile "${self}:  Log messages from ENS_EnsemblePost.ksh \
                    will be appended to ${logfile} ..." \
                    $self $scriptdir $logfile $email_target
   #----------------------------------------------------------------------
   # Note that $maxensemble is passed in place of $thisensemble in this case.  
   #----------------------------------------------------------------------
   AddJobStep $jobscript $ENS_Batch $scriptdir $step_num ${ENS_CONFIG_PostQueueName} \
              "serial" 1 ${scriptdir}/ENS_EnsemblePost.ksh ${jobType} \
              $ENS_CONFIG_EnsPostWallTime $maxensemble $maxensemble ${memberId[*]} \
              $ensemble_dir $InteractiveRun $logfile $email_target $config_tag \
              $ENS_JGEValidDate ${statusFlag[$ensCount]} $MassStore 
   (( ensCount += 1 ))
done

#----------------------------------------------------------------------
# submit multi-step job to LoadLeveler/LSF
#----------------------------------------------------------------------
if [[ $ENS_Batch = "LSF" ]] ; then
  chmod a+x ${jobscript} || \
    ErrorEmailExit "failed to chmod job script ${jobscript}" \
      $self $logfile $email_target $ENS_JGEValidDate
  AppendToLogFile "${self}:  Running ${jobscript} to submit LSF jobs..." \
    $self $scriptdir $logfile $email_target
  jobid=$( ${jobscript} ) || \
    ErrorEmailExit "failed to execute ${jobscript}" \
      $self $logfile $email_target $ENS_JGEValidDate
elif [[ $ENS_Batch = "LOADLEVELER" ]] ; then
  AppendToLogFile "${self}:  Submitting LoadLeveler Job Command File ${jobscript} ..." \
    $self $scriptdir $logfile $email_target
  jobid=$( llsubmit ${jobscript} | grep submitted | ${scriptdir}/get_jobid.ksh $scriptdir $logfile $email_target $ENS_JGEValidDate ) || \
    ErrorEmailExit "failed to parse job id" \
      $self $logfile $email_target $ENS_JGEValidDate
else
  chmod a+x ${jobscript} || \
    ErrorEmailExit "failed to chmod job script ${jobscript}" \
      $self $logfile $email_target $ENS_JGEValidDate
  AppendToLogFile "${self}:  Submitting Job Command File ${jobscript} ..." \
    $self $scriptdir $logfile $email_target
  jobid=$( ${jobscript} | grep submitted | ${scriptdir}/get_jobid.ksh $scriptdir $logfile $email_target $ENS_JGEValidDate ) || \
    ErrorEmailExit "failed to parse job id" \
      $self $logfile $email_target $ENS_JGEValidDate

fi

#----------------------------------------------------------------------
# echo LoadLeveler jobid or LSF group name to caller
#----------------------------------------------------------------------
print "${jobid}"

exit 0

