#!/bin/ksh
#
# Script Name:
#
#   ENS_MemberPost.ksh
#
# Usage:  
#
#   ENS_MemberPost.ksh $scriptdir $thisensemble $ensemble_dir $ENS_ValidDate \
#                      $write_netcdf_to_mss $logfile $email_target
#
#     $scriptdir           absolute path to these scripts
#     $thisensemble        ensemble number
#     $ensemble_dir        top-level working directory
#     $ENS_ValidDate       valid start date of forecast
#     $write_netcdf_to_mss skip wrfpost and write netcdf WRF output files to 
#                          MSS if TRUE (NCAR only)
#     $logfile             full path to logfile
#     $email_target        where to send email if something fails
#     $congig_tag          posterior tag on configs and namelists, e.g. DEMO
#
# Description:  
#
#   Ensemble member post-processing.  
#   Post-processes all wrfout files for this ensemble member or move them to 
#   MSS (MSS at NCAR only).  
#
#   If post-processing:  
#   Runs post.ksh to convert from netCDF terrain-following fields to grib1 
#   pressure/height fields.  
#   Post-processed results are stored in 
#   ${ensemble_dir}/${ENS_CONFIG_RunPath}/postprd_${wrf_domain}/ where 
#   $wrf_domain is d01 or d02 or d03 (etc.).  This convention is 
#   defined in post.ksh.  
#
# Dependencies:  
#
#   Since this script is intended to be called from a script called from cron, 
#   it has no dependencies on environment variables.  
#
#   Scripts called by this script must reside in the same directory as 
#   this script.  
#
# Author:  
#
#   Tom Henderson                          email:  hender@ucar.edu
#   NCAR, MMM Division                     voice:  303 497 1628
#   3450 Mitchell Lane                       fax:  303 497 8181
#   Boulder, Colorado 80301-2260 USA
#

set -u

# name of this script
self=$(/usr/bin/basename $0)

# load arguments
scriptdir=$1; shift
thisensemble=$1; shift
ensemble_dir=$1; shift
ENS_ValidDate=$1; shift
write_netcdf_to_mss=$1; shift
logfile=$1; shift
email_target=$1; shift
config_tag=$1; shift

# explicitly load all shared functions
. ${scriptdir}/ENS_SharedFunctions.ksh
# explicitly set path
SetPath

AppendToLogFile "${self}:  WRF POST BEGIN for member ${thisensemble} at $(date -u)" \
  $self $scriptdir $logfile $email_target

if [[ $write_netcdf_to_mss = "TRUE" ]] ; then
  AppendToLogFile "${self}:  Skipping wrfpost and moving netcdf output to MSS" \
    $self $scriptdir $logfile $email_target
fi

# Get site and exit if not AFWA or NCAR
GetSite
if [[ $ENS_Site != "AFWA" && $ENS_Site != "NCAR" ]] ; then
  AppendToLogFile "${self}:  No postprocessing specified at $(date -u)" \
  $self $scriptdir $logfile $email_target
  return
fi

# parse config file for $thisensemble
# GetMyConfig sets variables named ENS_CONFIG_*
GetMyConfig $thisensemble $scriptdir $logfile $email_target $config_tag

# turn on for debugging
if [[ $ENS_CONFIG_DEBUG = "TRUE" ]] ; then
  set -x
fi

runpath="${ensemble_dir}/${ENS_CONFIG_RunPath}"
cd ${runpath} || \
  ErrorEmailExit "POST_FAIL:  Could not cd to ${runpath}" \
    $self $logfile $email_target $ENS_ValidDate

#----------------------------------------------------------------------
# look for "SUCCESS" message in wrf.exe output files and exit on failure
# Note that this is redundant with ENS_WRF_Model.ksh because LSF may
# cause scripts to die in the middle!!!
#----------------------------------------------------------------------
wrf_exec=${ENS_CONFIG_ExecPath}/wrf.exe
if [[ $ENS_MPILauncher ]] ; then
  CheckRSLOutput ${runpath} ${self} ${wrf_exec} ${thisensemble} \
                 ${scriptdir} ${logfile} ${email_target} $ENS_ValidDate
else
  CheckRegularLogs ${runpath} ${self} ${wrf_exec} ${thisensemble} \
                 ${scriptdir} ${logfile} ${email_target} $ENS_ValidDate \
                 wrf.log
fi

# Post-process all wrfout files for this ensemble member.  
# Run post.ksh to convert from netCDF terrain-following fields to grib1 
# pressure/height fields.  
AppendToLogFile "${self}:  Begin post-processing member ${thisensemble}..." \
  $self $scriptdir $logfile $email_target
#$$$  TODO:  get theater from config file
AFWA_theater="t64"
# loop over WRF domains
if (( ENS_CONFIG_NumDomains > 9 )) ; then
  ErrorEmailExit "POST_FAIL:  ENS_CONFIG_NumDomains must be < 10" \
    $self $logfile $email_target $ENS_ValidDate
fi
logfile_post="${logfile}.${thisensemble}.post"
thisdomain=1
until (( thisdomain > ENS_CONFIG_NumDomains )) ; do
  wrf_domain="d0${thisdomain}"
  AppendToLogFile "${self}:  Post-processing domain ${wrf_domain}..." \
    $self $scriptdir $logfile $email_target
  fcst_hr=0
  for wrfoutfile in wrfout_${wrf_domain}_* ; do
    AppendToLogFile "${self}:  Post-processing member ${thisensemble} file ${wrfoutfile}, fcst_hr = ${fcst_hr}..." \
      $self $scriptdir $logfile $email_target
    # check wrfout file for NaNs...  
    ncdump ${wrfoutfile} | grep -v DOMINANT | grep -i nan > /dev/null 2>&1
    if (( $? == 0 )) ; then
      if [[ $write_netcdf_to_mss = "TRUE" ]] ; then
        ErrorEmailExit "POST_FAIL:  found NaNs in member ${thisensemble} file ${wrfoutfile}" \
          $self $logfile $email_target $ENS_ValidDate
      else
        AppendToLogFile "POST_FAIL:  found NaNs in member ${thisensemble} file ${wrfoutfile}, continuing" \
          $self $scriptdir $logfile $email_target
      fi
    else
      if [[ $write_netcdf_to_mss = "FALSE" ]] ; then
        wrf_timestr=${wrfoutfile#wrfout_${wrf_domain}_}
        # call my modified version of the post.ksh script (Chris Harrop et. al.) ...
        AppendToLogFile "${self}:  ${scriptdir}/post.ksh ${scriptdir} ${thisensemble} ${ensemble_dir} ${ENS_ValidDate} ${fcst_hr} ${wrf_timestr} ${wrf_domain} ${AFWA_theater} ${logfile} ${email_target}" \
          $self $scriptdir $logfile $email_target
        print "BEGIN LOG FOR post.ksh ENSEMBLE MEMBER ${thisensemble} DOMAIN ${wrf_domain}" >> ${logfile_post}
        ${scriptdir}/post.ksh ${scriptdir} ${thisensemble} ${ensemble_dir} \
          ${ENS_ValidDate} ${fcst_hr} ${wrf_timestr} ${wrf_domain} \
          ${AFWA_theater} ${logfile} ${email_target} >> ${logfile_post} 2>&1
        post_status=$?
        print "END LOG FOR post.ksh ENSEMBLE MEMBER ${thisensemble} DOMAIN ${wrf_domain}" >> ${logfile_post}
        cat $logfile_post >> $logfile
        # Continue to the next file if post.ksh fails...  
        if (( $post_status != 0 )) ; then
          AppendToLogFile "POST_FAIL:  post.ksh failed for member ${thisensemble} file ${wrfoutfile}, see log file ${logfile}, continuing" \
            $self $scriptdir $logfile $email_target
        else
          \rm -f $logfile_post
        fi
      fi
    fi
    (( fcst_hr += ENS_CONFIG_HistoryInterval ))
  done
  (( thisdomain += 1 ))
done

# Move files to MSS (NCAR only)
if [[ $write_netcdf_to_mss = "TRUE" ]] ; then
  AppendToLogFile "${self}:  WRF MEMBER POST Begin MSS transfer of data files for member ${thisensemble} at $(date -u)" \
    $self $scriptdir $logfile $email_target
  lsfile="ls.Member${thisensemble}"
  \rm -f $lsfile
  ls -lrRt > $lsfile
  tarfile="wrfout_Member${thisensemble}.tar"
# TODO:  warning -- duplication!
  tarcmd="tar cf ${tarfile} ${lsfile} LANDUSE.TBL ungrib.log metgrid.log.* wrf*_d0* WRF*.out WRF*.err rsl.*.* namelist*"
# TODO:  put this in a function & remove duplication with ENS_EnsemblePost.ksh
  AppendToLogFile "$tarcmd" $self $scriptdir $logfile $email_target
  tar cf ${tarfile} ${lsfile} LANDUSE.TBL ungrib.log metgrid.log.* wrf*_d0* WRF*.out WRF*.err rsl.*.* namelist* || \
    ErrorEmailExit "POST_FAIL:  error tarring WRF model output files" \
      $self $logfile $email_target $ENS_ValidDate
# TBH:  Removed gzip for now since it is really slow for long runs.  
  msscmd="msrcp -period 3650 -project $ENS_CONFIG_Account -srcdelete ${tarfile} ${ENS_CONFIG_MSSPath}/${ENS_ValidDate}/${tarfile}"
  AppendToLogFile "$msscmd" $self $scriptdir $logfile $email_target
  msrcp -period 3650 -project $ENS_CONFIG_Account -srcdelete ${tarfile} ${ENS_CONFIG_MSSPath}/${ENS_ValidDate}/${tarfile} || \
    ErrorEmailExit "POST_FAIL:  Could not transfer ${tarfile} to MSS" \
      $self $logfile $email_target $ENS_ValidDate
  AppendToLogFile "${self}:  WRF MEMBER POST End MSS transfer of data files for member ${thisensemble} at $(date -u)" \
    $self $scriptdir $logfile $email_target
fi

AppendToLogFile "${self}:  WRF POST END for member ${thisensemble} at $(date -u)" \
  $self $scriptdir $logfile $email_target

exit 0

