#!/bin/ksh
#==========================================================================
#
# Script Name:
#
#   ENS_Obsproc.ksh
# TODO:  How to determine if da_run_obsproc.ksh was successful
# TODO:  Put ENS_CONFIG_* variables in config_Ensemble file
# TODO:  Get input data files for SEA domain, or do ob.ascii files already exist?
# Usage:  
#
#   ENS_Obsproc.ksh $scriptdir $ensemble_dir \
#                   $ENS_ValidDate $logfile $email_target
# Example: ENS_Obsproc.ksh \ 
#          /home/bluevista/schramm/JEFS_2.2/ENS/ENS_scripts \
#          /ptmp/schramm/ENS_Ensembles/2007032300  2007032300 \
#          /ptmp/schramm/ENS_Ensembles/2007032300/obsproc.log schramm@ucar.edu
#Should be  /ptmp/schramm/ENS_Ensembles/logs/YYYYMMDD_timestamp/ENS_Obsproc.log
#
# Input data file:  ${OB_DIR}/obs.${ENS_ValidDate}
#
#     $scriptdir           absolute path to these scripts
#     $ensemble_dir        top-level working directory (includes date dir)
#     $ENS_ValidDate       valid start date of forecast
#     $logfile             full path to logfile
#     $email_target        where to send email if something fails
#
# Description:  
#
#   obsproc pre-processes observations for subsequent use in WRF-Var.  This
#   is member-independent, but must be run at the beginning of each cycle.
#   This driver script runs the obsproc script da_run_obsproc.ksh as a black
#   box and checks for successful completion.  da_run_obsproc.ksh generates
#   a namelist and runs the obsproc executable.
#
# Dependencies:  
#
#   Since this script is intended to be called from a script called from cron, 
#   it has no dependencies on environment variables.  
#
#   Scripts called by this script must reside in the same directory as 
#   this script.  
#
# Author:  
#
#   Tom Henderson                          email:  hender@ucar.edu
#     NCAR, MMM Division                     voice:  303 497 1628
#     3450 Mitchell Lane                       fax:  303 497 8181
#     Boulder, Colorado 80301-2260 USA
#   Julie Schramm                          email:  schramm@ucar.edu
#     NCAR, MMM Division                   voice:  303 497 8191
#     3450 Mitchell Lane                     fax:  303 497 8181
#     Boulder, Colorado 80301-2260 USA
#
#==========================================================================

# name of this script
self=$(/usr/bin/basename $0)

#-------------------------------------------------------------------------
# load arguments
#-------------------------------------------------------------------------
scriptdir=$1; shift
ensemble_dir=$1; shift
ENS_ValidDate=$1; shift
logfile=$1; shift
email_target=$1; shift

#-------------------------------------------------------------------------
# explicitly load all shared functions
#-------------------------------------------------------------------------
. ${scriptdir}/ENS_SharedFunctions.ksh

#-------------------------------------------------------------------------
# explicitly set path
#-------------------------------------------------------------------------
SetPath

AppendToLogFile "${self}:  BEGIN obsproc for  ${ENS_ValidDate} at $(date -u)" \
  $self $scriptdir $logfile $email_target

#-------------------------------------------------------------------------
# parse ensemble-wide config file
# GetMyConfig sets variables named ENS_CONFIG_*
#-------------------------------------------------------------------------
GetMyConfig 0 $scriptdir $logfile $email_target

#----------------------------------------------------------------------
# Set export variables from CONFIG vars
#----------------------------------------------------------------------
#export CLEAN=$ENS_CONFIG_CLEAN               # Remove work dirs if true
#export EXPT=$ENS_CONFIG_EXPT_NAME            # Experiment name
#export DATE=$ENS_ValidDate                   # Current date
#export MAX_OB_RANGE=$ENS_CONFIG_MAX_OB_RANGE # Range about DATE to get obs (hr)
#export FCST_RANGE=$ENS_CONFIG_FCST_RANGE     # Cycle period, not used in obsproc
#export REL_DIR=$ENS_CONFIG_DATC_TOP          # Top level for DATC code
#export WRFVAR_DIR=$ENS_CONFIG_WRFVAR_TOP     # Top level for WRF-Var code

export CLEAN=false                          # Remove work dirs if true
export EXPT="ENS"                           # Experiment name
export DATE=$ENS_ValidDate                  # Current date
export MAX_OB_RANGE=2                       # Range about DATE to get obs (hr)
export FCST_RANGE=6                         # Cycle period, not used in obsproc
export REL_DIR=/home/bluevista/schramm/code # Top level for DATC code
export WRFVAR_DIR=$REL_DIR/wrfvar           # Top level for WRF-Var code

export DAT_DIR=$ensemble_dir/obsproc        # Top level for obsproc output
#----------------------------------------------------------------------
# These are set to avoid using default directories in da_run_obsproc.ksh
#----------------------------------------------------------------------
export OBSPROC_DIR=$REL_DIR/3DVAR_OBSPROC   # Obsproc exe
export OB_DIR=$DAT_DIR/data                 # Output Obsproc dir 
export RTOBS_DIR=/ptmp/schramm/data/rtobs   # Input Obsproc dir
export REG_DIR=$DAT_DIR                     # Top dir for regional output 
export EXP_DIR=$DAT_DIR                     # Top dir for particular exp. 
export RUN_DIR=$DAT_DIR                     # Run dir for experiment 
export WORK_DIR=$RUN_DIR/working            # Work directory , used by CLEAN 
                                            # NOTE: No default in obsproc
#----------------------------------------------------------------------
# Namelist variables used in WPS namelist
#----------------------------------------------------------------------
#export NL_E_WE=$ENS_CONFIG_E_SN         # Each nest's W-E dimension, integers
#export NL_E_SN=$ENS_CONFIG_E_SN         # Each nest's S-N dimension, integers
#export MAP_PROJ=$ENS_CONFIG_MAP_PROJ    # Map projection
#export REF_LAT=$ENS_CONFIG_REF_LAT      # Known latitude in coarse domain
#export REF_LON=$ENS_CONFIG_REF_LON      # Known long. in simulation domain
#export TRUELAT1=$ENS_CONFIG_TRUELAT1    # True latitude 1
#export TRUELAT2=$ENS_CONFIG_TRUELAT2    # True latitude 2
#export STAND_LON=$ENS_CONFIG_STAND_LON  # Std lon of the working domain
#export NL_DX=$ENS_CONFIG_DX             # Grid distance in x-direction

# CONUS values for testing
export NL_E_WE=45         # Each nest's W-E dimension, integers
export NL_E_SN=45         # Each nest's S-N dimension, integers
export MAP_PROJ=lambert   # Map projection
export REF_LAT=40.0      # Known latitude in coarse domain
export REF_LON=-98.0     # Known long. in simulation domain
export TRUELAT1=30.0      # True latitude 1
export TRUELAT2=60.0      # True latitude 2
export STAND_LON=-98.0    # Std lon of the working domain
export NL_DX=200000

#----------------------------------------------------------------------
# MM5 namelist variables (not used in WRF)
#----------------------------------------------------------------------
export PS0=100000.0         # Reference sea level pressure
export TS0=300.0            # Reference sea level temperature
export TLP=50.0             # Temperature lapse rate

ERR='<FONT COLOR="red">'
END='</FONT>'
#----------------------------------------------------------------------
# Set the directories needed from CONFIG variables
#----------------------------------------------------------------------
mkdir -p $DAT_DIR || ErrorEmailExit "could not create obsproc run directory \
                       for ensemble member ${ENS_CONFIG_MemberName}" \
                       $self $logfile $email_target

#----------------------------------------------------------------------
# Trace the script usage
#----------------------------------------------------------------------
$WRFVAR_DIR/scripts/da_trace.ksh da_run_obsproc $DAT_DIR

#----------------------------------------------------------------------
# Check for existence of compressed input data file
#----------------------------------------------------------------------
obs_file=${RTOBS_DIR}/${ENS_ValidDate}/obs.${ENS_ValidDate}.gz

if [[ ! -e $obs_file ]]; then
       errMessage="The file ${obs_file} does not exist."
       ErrorEmailExit "$errMessage" $self $logfile $email_target $ENS_ValidDate
fi

#----------------------------------------------------------------------
# Run DATC da_run_obsproc.ksh script
#----------------------------------------------------------------------
${WRFVAR_DIR}/scripts/da_run_obsproc.ksh > $DAT_DIR/index.html 2>&1

#----------------------------------------------------------------------
# Do lots of error checking after, since there is none in black box
#----------------------------------------------------------------------
obsproc_status=$?
if (( $obsproc_status != 0 )) ; then
  ErrorEmailExit "da_run_obsproc.ksh failed, see log file ${logfile}" \
    $self $logfile $email_target
fi

AppendToLogFile "${self}: END LOG FOR ENS_Obsproc.ksh for cycle ${ENS_ValidDate}" \
                 $self $scriptdir $logfile $email_target

#----------------------------------------------------------------------
# Need a "SUCCESS" message in output files to determine failure
#----------------------------------------------------------------------

AppendToLogFile "${self}: END obsproc for cycle ${ENS_ValidDate} at $(date -u)" \
                 $self $scriptdir $logfile $email_target

#----------------------------------------------------------------------
# Begin cleanup
#----------------------------------------------------------------------
if $CLEAN; then rm -rf $WORK_DIR; fi

#----------------------------------------------------------------------
# Function to unset export variables
#----------------------------------------------------------------------
unset CLEAN
unset EXPT
unset DATE
unset MAX_OB_RANGE
unset FCST_RANGE
unset REL_DIR
unset WRFVAR_DIR
unset DAT_DIR
unset REG_DIR
unset EXP_DIR
unset RUN_DIR
unset WORK_DIR
unset OB_DIR
unset NL_E_WE
unset NL_E_SN
unset MAP_PROJ
unset REF_LAT
unset REF_LON
unset TRUELAT1
unset TRUELAT2
unset STAND_LON
unset NL_DX
unset PS0
unset TS0
unset TLP

exit 0

