#!/bin/ksh
#
# Description:  
#
#   Functions shared by other ENS scripts.  
#
#   Use ". ENS_SharedFunctions.ksh" to explicitly load functions.  
#
# Dependencies:  
#
#   None.  
#
# Authors:  
#
#   Tom Henderson                          email:  hender@ucar.edu
#   NCAR, MMM Division                     voice:  303 497 1628
#   3450 Mitchell Lane                       fax:  303 497 8181
#   Boulder, Colorado 80301-2260 USA
#
#   Julie Schramm                          email:  schramm@ucar.edu
#   NCAR, MMM Division                     voice:  303 497 8191
#   3450 Mitchell Lane                       fax:  303 497 8181
#   Boulder, Colorado 80301-2260 USA
#
#   Josh Hacker                            email:  hacker@ucar.edu
#   NCAR RAL Division                      voice:  303 497 8188
#   3450 Mitchell Lane
#   Boulder, Colorado 80301-2260 USA
#

# Print error message and exit.  
# Usage:
#   ErrorExit error_message script_name
function ErrorExit {
  typeset error_message=$1
  typeset script_name=$2
  print "ERROR ${script_name}:  ${error_message}"
  exit 20
}


# Set ENS_Site to either local machine - don't want any real dependence
# is being run.  
# Usage:
#   GetSite
# Return values:
#   ENS_Site is set to "AFWA" if script is being run at AFWA.  
#   ENS_Site is set to "NCAR" otherwise.  
#
function GetSite {
  ENS_Site=`hostname`
}


# Set ENS_Batch to either "LOADLEVELER" or "LSF".  
# At present $ENS_Batch depends only upon where this script 
# is being run.  
# Usage:
#   GetBatch
# Return values:
#   ENS_Batch is set to "LOADLEVELER" if script will use LoadLeveler.  
#   ENS_Batch is set to "LSF" if script will use LSF.  
#
function GetBatch {
  GetSite  # sets ENS_Site
  if [[ $ENS_Site = "NCAR" ]] ; then
    # at NCAR
    ENS_Batch="LSF"
  elif [[ $ENS_Site = "AFWA" ]]; then
    # at AFWA
    ENS_Batch="LOADLEVELER"
  else
    ENS_Batch="NO_QUEUE"
  fi
}


# Set ENS_MPILauncher to either "poe" or "mpirun.lsf".  
# Usage:
#   GetMPILauncher
# Return values:
#   ENS_MPILauncher is set to "poe" if script will use LoadLeveler.  
#   ENS_MPILauncher is set to "mpirun.lsf" if script will use LSF.  
#
function GetMPILauncher {
  GetBatch  # sets ENS_Batch
  if [[ $ENS_Batch = "LSF" ]] ; then
    ENS_MPILauncher="$(which mpirun.lsf)"
  elif [[ $ENS_Batch = "LOADLEVELER" ]] ; then
    ENS_MPILauncher="poe"
  else
    ENS_MPILauncher=""
  fi
}


# Return user name.  
# Usage:
#   GetUser
# Return values:
#   ENS_User is set to "$(whoami)".  
#
function GetUser {
  GetSite  # sets ENS_Site
  ENS_User="$(whoami)"
  ENS_USER=$(echo $ENS_User | tr '[a-z]' '[A-Z]')
}


# Set path for either "AFWA" or "NCAR" depending on where this script 
# is being run.  
# Usage:
#   SetPath
# Return values:
#   $PATH is set for "AFWA" if script is being run at AFWA.  
#   $PATH is set for "NCAR" otherwise.  
#
function SetPath {
  # set ENS_Site
  GetSite  # sets ENS_Site
  if [[ $ENS_Site = "NCAR" ]] ; then
    # at NCAR
    export PATH=.:/usr/local/lsf/6.2/aix5-64/etc:/usr/local/lsf/6.2/aix5-64/bin:/usr/vac/bin:/usr/vacpp/bin:/usr/bin:/usr/ucb:/usr/bin/X11:/usr/local/dcs-4.0/bin:/usr/local/bin:/usr/lpp/xlf/lib:/usr/lpp/xlf/bin:/ssg/loadl/bin:/usr/sbin:/contrib/bin:/usr/local/bin
  elif [[ $ENS_Site = "AFWA" ]] ; then
    # at AFWA
    export PATH=.:/usr/local/ossh/bin:/usr/bin:/usr/ucb:/usr/lib/instl:/usr/sbin:/usr/bin:/opt/csm/bin:/usr/local/bin:/usr/lpp/ssp/bin:/usr/lpp/ssp/install/bin:/usr/lpp/ssp/kerberos/bin:/usr/lpp/ssp/rcmd/bin:/usr/lpp/LoadL/full/bin:/usr/vac/bin:/usr/local/netcdf/bin:/usr/vacpp/bin:/bin:/usr/opt/ifor/ls/os/aix/bin
  else # use existing path
    export PATH=$PATH
  fi
}



# Append message to log file.  
# if append fails, attempt to send email and exit
# Usage:
#   AppendToLogFile message script_name script_dir logfile email_address
function AppendToLogFile {
  typeset message=$1
  typeset script_name=$2
  typeset script_dir=$3
  typeset logfile=$4
  typeset email_address=$5
  if [[ ! -f $logfile ]] ; then
    touch $logfile  # if this fails, the following print will fail too
  fi
  print $message >> $logfile
  typeset stat=$?
  # note:  the following bit is not completely safe...  
  # note:  need to throw an exception...  
  if (( $stat != 0 )) ; then
    if [[ ${ENS_CONFIG_EmailNotify} = "yes" ]] ; then
      mail -s "ENS ERROR ${script_name}:  could not write to log file ${logfile}" $email_address < ${script_dir}/LogFileFail.msg
      mail -s "ENS ERROR ${script_name}:  could not write to log file ${logfile}, here it is..." $email_address < ${logfile}
      exit $stat
    fi
  fi
}



# Send log file to specified address with error message as subject line and 
# exit.  
# Usage:
#   ErrorEmailExit error_message script_name logfile email_address valid_date
function ErrorEmailExit {
  typeset error_message=$1
  typeset script_name=$2
  typeset logfile=$3
  typeset email_address=$4
  typeset valid_date=$5
  typeset msg="ERROR ${script_name} ${valid_date} FAIL:  ${error_message}"
  # attempt to append message to the logfile
  print $msg >> $logfile
  if [[ ${ENS_CONFIG_EmailNotify} = "yes" ]] ; then
    mail -s "ENS ERROR:  [${msg}]" $email_address < ${logfile}
  else
    # if no email available, just print error message
    print $msg
  fi
  exit 30
}



# Get file from src to dest with error exit on failure.  
# For now, use cp instead of ln since we do not know if files 
# will be on the same file system (if not, "ln -s" fails).  
# Usage:
#   GetFile src dest logfile email_address valid_date
function GetFile {
  typeset src=$1
  typeset dest=$2
  typeset logfile=$3
  typeset email_address=$4
  typeset valid_date=$5
  cp ${src} ${dest} || \
    ErrorEmailExit "Failed to copy ${src} to ${dest}" \
      "GetFile" $logfile $email_address $valid_date
}



# Copy file from src to dest with error exit on failure.  
# Overwrite dest if present.  
# Usage:
#   CopyFileF src dest logfile email_address valid_date
function CopyFileF {
  typeset src=$1
  typeset dest=$2
  typeset logfile=$3
  typeset email_address=$4
  typeset valid_date=$5
  /bin/cp -f ${src} ${dest} || \
    ErrorEmailExit "Failed to copy ${src} to ${dest}" \
      "CopyFileF" $logfile $email_address $valid_date
}



# Move file from src to dest with error exit on failure.  
# Usage:
#   MoveFile src dest logfile email_address valid_date
function MoveFile {
  typeset src=$1
  typeset dest=$2
  typeset logfile=$3
  typeset email_address=$4
  typeset valid_date=$5
  mv ${src} ${dest} || \
    ErrorEmailExit "Failed to move ${src} to ${dest}" \
      "MoveFile" $logfile $email_address $valid_date
}




# Move file from src to dest with error exit on failure.  
# Overwrite dest if present.  
# Usage:
#   MoveFileF src dest logfile email_address valid_date
function MoveFileF {
  typeset src=$1
  typeset dest=$2
  typeset logfile=$3
  typeset email_address=$4
  typeset valid_date=$5
  /bin/mv -f ${src} ${dest} || \
    ErrorEmailExit "Failed to move ${src} to ${dest}" \
      "MoveFileF" $logfile $email_address $valid_date
}



# Get (i.e. copy or link -- whatever GetFile does) WRF tables from directory 
# $src to directory $dest.  
# Assumes files do not already exist in $dest.  
# LANDUSE.TBL is skipped.  
# Usage:
#   GetWRFTables src dest logfile email_address valid_date
function GetWRFTables {
  typeset src=$1
  typeset dest=$2
  typeset logfile=$3
  typeset email_address=$4
  typeset valid_date=$5
  set -A staticfiles SOILPARM.TBL VEGPARM.TBL gribmap.txt tr49t67 tr49t85 tr67t85 ETAMPNEW_DATA GENPARM.TBL README.namelist RRTM_DATA CAM_ABS_DATA CAM_AEROPT_DATA ozone.formatted ozone_lat.formatted ozone_plev.formatted urban_param.tbl
  for sfile in ${staticfiles[*]} ; do
    GetFile "${src}/${sfile}" ${dest} $logfile $email_address $valid_date
  done
}



# Move WRF tables from directory $src to directory $dest.  
# Assumes files do not already exist in $dest.  
# LANDUSE.TBL is skipped.  
# Usage:
#   MoveWRFTables src dest logfile email_address valid_date
function MoveWRFTables {
  typeset src=$1
  typeset dest=$2
  typeset logfile=$3
  typeset email_address=$4
  typeset valid_date=$5
  set -A staticfiles SOILPARM.TBL VEGPARM.TBL gribmap.txt tr49t67 tr49t85 tr67t85 ETAMPNEW_DATA GENPARM.TBL README.namelist RRTM_DATA
  for sfile in ${staticfiles[*]} ; do
    MoveFile "${src}/${sfile}" ${dest} $logfile $email_address $valid_date
  done
}



# Read a configuration file and parse out keys and values, then store values in 
# non-local variables named ENS_CONFIG_* that can be accessed by the caller.  
# See ../ENS_config/README.configfiles for a detailed description.  
# Also, the configuration file is copied into the log file directory if a 
# copy of the file is not already there.  
# Arguments are:  
#   configfilename Full path to configuration file.  
#   logfile        Name of logfile for writing error messages.  
#   email_address  Email address to which error messages may be sent (NCAR 
#                  only).  
# Usage:
#   GetConfig configfilename logfile email_address
function GetConfig {
  typeset configfilename=$1
  typeset logfile=$2
  typeset email_address=$3

  typeset script_name="GetConfig"

  # read file into array filtering out comment lines
  set -A configlines $(cat $configfilename | grep -v ^#)
  if (( $? != 0 )) ; then
    ErrorEmailExit "Failed to read config file ${configfilename}" \
      $script_name $logfile $email_address ""
  fi

  # parse array into key and value arrays
  typeset numlines=${#configlines[*]}
  typeset thisline=0
  typeset thispair=0
  until (( thisline >= numlines )) ; do
    if [[ ${configlines[(( thisline+1 ))]} = "::" ]] ; then
      typeset keys[thispair]=${configlines[$thisline]}
      typeset values[thispair]=${configlines[(( thisline+2 ))]}
      (( thispair += 1 ))
      (( thisline += 3 ))
    else
      ErrorEmailExit "Corrupt config file, error encountered attempting to parse key=<${configlines[$thisline]}>, value=<${configlines[(( thisline+2 ))]}>" \
        $script_name $logfile $email_address ""
    fi
  done

  # set all config variables
  # these variables are NOT local to this function
  GetUser  # sets ENS_User
  typeset numpairs=${#keys[*]}
  typeset thispair=0
  until (( thispair >= numpairs )) ; do
    typeset thiskey=${keys[$thispair]}
    typeset thisvalue=${values[$thispair]}
    # convert string "ENS_User" to actual $ENS_User
    thisvalue=$(echo $thisvalue | sed -e "s/ENS_User/${ENS_User}/")
    # convert string "ENS_USER" to actual $ENS_USER
    thisvalue=$(echo $thisvalue | sed -e "s/ENS_USER/${ENS_USER}/")
    case "$thiskey" in
      ENS_CONFIG_MemberName)
        ENS_CONFIG_MemberName=$thisvalue
        ;;
      ENS_CONFIG_RunPath)
        ENS_CONFIG_RunPath=$thisvalue
        ;;
      ENS_CONFIG_ExecPath)
        ENS_CONFIG_ExecPath=$thisvalue
        ;;
      ENS_CONFIG_RealExecPath)
        ENS_CONFIG_RealExecPath=$thisvalue
        ;;
# JPH new WPS key::values
      ENS_CONFIG_WPSRoot)
        ENS_CONFIG_WPSRoot=$thisvalue
        ;;
      ENS_CONFIG_WPSVtableGFS)
        ENS_CONFIG_WPSVtableGFS=$thisvalue
        ;;
      ENS_CONFIG_WPSVtableSST)
        ENS_CONFIG_WPSVtableSST=$thisvalue
        ;;
      ENS_CONFIG_WPSVtableAGRMET)
        ENS_CONFIG_WPSVtableAGRMET=$thisvalue
        ;;
      ENS_CONFIG_FGIntervalHrs)
        ENS_CONFIG_FGIntervalHrs=$thisvalue
        ;;
      ENS_CONFIG_FGNumLevels)
        ENS_CONFIG_FGNumLevels=$thisvalue
        ;;
      ENS_CONFIG_WPSGeoPath)
        ENS_CONFIG_WPSGeoPath=$thisvalue
        ;;
      ENS_CONFIG_g1print)
        ENS_CONFIG_g1print=$thisvalue
        ;;
# End JPH new WPS key::values
      ENS_CONFIG_Account)
        ENS_CONFIG_Account=$thisvalue
        ;;
      ENS_CONFIG_MSSPath)
        ENS_CONFIG_MSSPath=$thisvalue
        ;;
      ENS_CONFIG_NamelistPath)
        ENS_CONFIG_NamelistPath=$thisvalue
        ;;
      ENS_CONFIG_Namelist)
        ENS_CONFIG_Namelist=$thisvalue
        ;;
      ENS_CONFIG_ModelDataPath)
        ENS_CONFIG_ModelDataPath=$thisvalue
        ;;
      ENS_CONFIG_RealDataPath)
        ENS_CONFIG_RealDataPath=$thisvalue
        ;;
      ENS_CONFIG_LANDUSELocation)
        ENS_CONFIG_LANDUSELocation=$thisvalue
        ;;
      ENS_CONFIG_LANDUSEFile)
        ENS_CONFIG_LANDUSEFile=$thisvalue
        ;;
      ENS_CONFIG_NodeUsage)
        ENS_CONFIG_NodeUsage=$thisvalue
        ;;
      ENS_CONFIG_TasksPerNode)
        ENS_CONFIG_TasksPerNode=$thisvalue
        ;;
      ENS_CONFIG_RealNumNodes)
        ENS_CONFIG_RealNumNodes=$thisvalue
        ;;
      ENS_CONFIG_RealWallTime)
        ENS_CONFIG_RealWallTime=$thisvalue
        ;;
      ENS_CONFIG_RealQueueName)
        ENS_CONFIG_RealQueueName=$thisvalue
        ;;
      ENS_CONFIG_FcstLengthHrs)
        ENS_CONFIG_FcstLengthHrs=$thisvalue
        ;;
      ENS_CONFIG_HistoryInterval)
        ENS_CONFIG_HistoryInterval=$thisvalue
        ;;
      ENS_CONFIG_WRFNumNodes)
        ENS_CONFIG_WRFNumNodes=$thisvalue
        ;;
      ENS_CONFIG_WRFWallTime)
        ENS_CONFIG_WRFWallTime=$thisvalue
        ;;
      ENS_CONFIG_WRFQueueName)
        ENS_CONFIG_WRFQueueName=$thisvalue
        ;;
      ENS_CONFIG_WPSNumNodes)
        ENS_CONFIG_WPSNumNodes=$thisvalue
        ;;
      ENS_CONFIG_WPSWallTime)
        ENS_CONFIG_WPSWallTime=$thisvalue
        ;;
      ENS_CONFIG_WPSQueueName)
        ENS_CONFIG_WPSQueueName=$thisvalue
        ;;
      ENS_CONFIG_PostLevel)
        ENS_CONFIG_PostLevel=$thisvalue
        ;;
      ENS_CONFIG_MeanSprVars)
        ENS_CONFIG_MeanSprVars=$thisvalue
        ;;
      ENS_CONFIG_RandomVariance)
        ENS_CONFIG_RandomVariance=$thisvalue
        ;;
      ENS_CONFIG_RandomVars)
        ENS_CONFIG_RandomVars=$thisvalue
        ;;
      ENS_CONFIG_PertScript)
        ENS_CONFIG_PertScript=$thisvalue
        ;;
      ENS_CONFIG_PostNumNodes)
        ENS_CONFIG_PostNumNodes=$thisvalue
        ;;
      ENS_CONFIG_PostWallTime)
        ENS_CONFIG_PostWallTime=$thisvalue
        ;;
      ENS_CONFIG_PostQueueName)
        ENS_CONFIG_PostQueueName=$thisvalue
        ;;
      ENS_CONFIG_EnsPostWallTime)
        ENS_CONFIG_EnsPostWallTime=$thisvalue
        ;;
      ENS_CONFIG_NumMembers)
        ENS_CONFIG_NumMembers=$thisvalue
        ;;
      ENS_CONFIG_RunMembers)
        ENS_CONFIG_RunMembers=$thisvalue
        ;;
      ENS_CONFIG_NumDomains)
        ENS_CONFIG_NumDomains=$thisvalue
        ;;
      ENS_CONFIG_EmailNotify)
        ENS_CONFIG_EmailNotify=$thisvalue
        ;;
      ENS_CONFIG_JGEDataPath)
        ENS_CONFIG_JGEDataPath=$thisvalue
        ;;
      ENS_CONFIG_JGERetroDataPath)
        ENS_CONFIG_JGERetroDataPath=$thisvalue
        ;;
      ENS_CONFIG_JGEMember)
        ENS_CONFIG_JGEMember=$thisvalue
        ;;
      ENS_CONFIG_agrmetDataPath)
        ENS_CONFIG_agrmetDataPath=$thisvalue
        ;;
      ENS_CONFIG_navysstDataPath)
        ENS_CONFIG_navysstDataPath=$thisvalue
        ;;
      ENS_CONFIG_WRFPostExecPath)
        ENS_CONFIG_WRFPostExecPath=$thisvalue
        ;;
      ENS_CONFIG_GRIBOutPath)
        ENS_CONFIG_GRIBOutPath=$thisvalue
        ;;
      ENS_CONFIG_DEBUG)
        ENS_CONFIG_DEBUG=$thisvalue
        ;;
      *)
        ErrorEmailExit "Error:  unknown key <${thiskey}> in config file" \
          $script_name $logfile $email_address ""
        ;;
    esac
    (( thispair += 1 ))
  done

  # Finally, copy this configuration file into the log file directory as a 
  # record of what was run.  Only make the copy if the file is not already 
  # there.  
  log_path=$(/usr/bin/dirname ${logfile})
  target_file="${log_path}/$(/usr/bin/basename ${configfilename})"
  if [[ ! -f $target_file ]] ; then
    cp $configfilename $target_file
    if (( $? != 0 )) ; then
      ErrorEmailExit "Could not copy configuration file ${configfilename} to log directory ${log_path}" \
        $script_name $logfile $email_address ""
    fi
  fi
}


# Parse all config files needed by this ensemble member and store values in 
# non-local variables named ENS_CONFIG_* that can be accessed by the caller.  
# See ../ENS_config/README.configfiles for a detailed description.  
# Arguments are:  
#   member_id      Integer id of this ensemble member.  If member_id == 0 then 
#                  only the ensemble-wide configuration file is read.  
#   scriptdir      Directory in which the calling script resides.  
#   logfile        Name of logfile for writing error messages.  
#   email_address  Email address to which error messages may be sent (NCAR 
#                  only).  
# Usage:
#   GetMyConfig member_id scriptdir logfile email_address
function GetMyConfig {
  typeset member_id=$1
  typeset scriptdir=$2
  typeset logfile=$3
  typeset email_address=$4
  typeset config_tag=$5
  GetSite  # sets ENS_Site
  # parse ensemble-wide config file
  configfilename="${scriptdir}/../ENS_config/config_Ensemble.${config_tag}"
  # GetConfig sets variables named ENS_CONFIG_*
  GetConfig $configfilename $logfile $email_address
  if (( member_id > 0 )) ; then
    # parse config file for this ensemble member
    configfilename="${scriptdir}/../ENS_config/config_Member${member_id}.${config_tag}"
    # GetConfig sets variables named ENS_CONFIG_*
    GetConfig $configfilename $logfile $email_address
  fi
}


#
# $$$ need to add more error handling!  
#
# Append to multi-job-step script for submission to llsubmit (LoadLeveler) or 
# bsub (LSF).  
#
# Note that submission to LSF can optionally behave interactively via use of 
# the "bsub -K" option.  This allows callers to avoid the complexity of 
# polling LSF to find out if jobs have finished yet.  This behavior is 
# controlled by $interactive_run.  
#
# Usage:
#   AddJobStep $jobscript $batch $scriptdir $step_num $class $job_type $node \
#                $executable $jmejobtype $wall_clock_limit $thisensemble \
#                $maxensemble ${memberId[*]} $ensemble_dir $interactive_run \
#                $logfile $email_target @allotherargs
#
# Arguments are:  
#  jobscript        Name of LoadLeveler or LSF Job Command File to append to.  
#  batch            Location of run:  "LOADLEVELER" or "LSF".  
#  scriptdir        Directory in which the calling script resides.  
#  step_num         Integer id of this job step.  
#  class            LoadLeveler/LSF class (queue name).  
#  job_type         LoadLeveler/LSF job_type ("serial" or "parallel").  
#  node             Number of LoadLeveler/LSF nodes.  Note that 
#                   tasks_per_node is defined via ENS_CONFIG_TasksPerNode.
#  executable       Full path to executable to run.  
#  jmejobtype       Type of ENS job.  Accepted values are "WRFWPS",
#                   "WRFREAL_WPS", 
#                   "WRFMODEL", "WRFPOST", "ENSPOST", and "ENSFAIL".  This 
#                   string is also used as the beginning of file names for 
#                   stdout and stderr files.  For each ensemble member, order 
#                   of job steps is WRFWPS, WRFREAL_WPS,
#                   WRFMODEL, WRFPOST.  
#                   ENSPOST follows once all WRFPOST are complete.  ENSFAIL is 
#                   only executed if ENSPOST is not called due to failure of 
#                   a previous job-step.  Note that ENSFAIL is not used for 
#                   LSF since it is not as capable as LoadLeveler.  
#  wall_clock_limit Wall-clock limit for this job step, as HH:MM.  
#  thisensemble     Ensemble member number, or number of ensemble members for 
#                   ENSPOST case.
#  maxensemble      Number of ensemble members.
#  ${memberId[*]}   Array of ensemble members to run.
#  ensemble_dir     Top-level working directory.
#  interactive_run  Run interactively (NCAR only).  
#  logfile          Full path to logfile.
#  email_target     Where to send email if something fails.
#  config_tag       Posterior tag on namelists and configs, e.g. DEMO
#  allotherargs     Any remaining arguments are optional and will be passed 
#                   down to $executable in the following order:  
#                     $scriptdir $thisensemble $ensemble_dir \
#                     ${allotherargs[*]} $logfile $email_target
#
# Dependencies: 
#  GetMyConfig must have already been called by the caller so ENS_CONFIG_* 
#  variables are valid.  
#
function AddJobStep {
#print "DEBUG:  hello from AddJobStep"
  typeset jobscript=$1; shift
  typeset batch=$1; shift
  typeset scriptdir=$1; shift
  typeset step_num=$1; shift
  typeset class=$1; shift
  typeset job_type=$1; shift
  typeset node=$1; shift
  typeset executable=$1; shift
  typeset jmejobtype=$1; shift
  typeset wall_clock_limit=$1; shift
  typeset thisensemble=$1; shift
  typeset maxensemble=$1; shift

  set -A memberId
  icount=0
  until (( icount >= maxensemble )) ; do
    memberId[$icount]=$1; shift
    (( icount += 1 ))
  done

  typeset ensemble_dir=$1; shift
  typeset interactive_run=$1; shift
  typeset logfile=$1; shift
  typeset email_target=$1; shift
  typeset config_tag=$1; shift

  typeset allotherargs=${*}

#print "DEBUG:  jobscript = <${jobscript}>"
#print "DEBUG:  step_num = <${step_num}>"
#print "DEBUG:  class = <${class}>"
#print "DEBUG:  job_type = <${job_type}>"
#print "DEBUG:  node = <${node}>"
#print "DEBUG:  executable = <${executable}>"
#print "DEBUG:  jmejobtype = <${jmejobtype}>"
#print "DEBUG:  thisensemble = <${thisensemble}>"
#print "DEBUG:  memberId = <${memberId[*]}>"
#print "DEBUG:  ensemble_dir = <${ensemble_dir}>"
#print "DEBUG:  logfile = <${logfile}>"
#print "DEBUG:  email_target = <${email_target}>"
#print "DEBUG:  allotherargs = <${allotherargs}>"

# $$$ add error check for allowed values of $jmejobtype, etc.

  # derive a few things from the arguments
  # Note that $maxensemble is passed in place of $thisensemble for ENSPOST and 
  # ENSFAIL.
  if [[ $jmejobtype = "ENSPOST" ]] ; then
    typeset step_name="ENSPOST"
  elif [[ $jmejobtype = "ENSFAIL" ]] ; then
# TODO:  MAYBE get rid of ENSFAIL for LoadLeveler too since scripts are now 
# TODO:  smart enough to work around LSF issues anyway!  
# TODO:  OR MAYBE NOT because it works really well at AFWA and does not 
# TODO:  depend on reading "FAIL" messages out of log files...  
    # skip ENSFAIL for LSF (ugh)
    if [[ $batch = "LSF" ]] ; then
      return 0
    else
      typeset step_name="ENSFAIL"
    fi
  else
    typeset step_name="${jmejobtype}_${thisensemble}"
  fi
  if [[ $batch = "LSF" ]] ; then
    # for LSF, append pid to avoid collisions with other ENS jobs
    step_name="${step_name}.${$}"
  fi
  typeset node_usage=${ENS_CONFIG_NodeUsage}
  if [[ $job_type = "serial" ]] ; then
    node_usage="shared"
  fi
  typeset runpath="${ensemble_dir}/${ENS_CONFIG_RunPath}"
  if [[ $jmejobtype = "ENSPOST" || $jmejobtype = "ENSFAIL" ]] ; then
    typeset initdir=${runpath}/..
  else
    typeset initdir=${runpath}
  fi

  # LoadLeveler allows a single big script
  # LSF wants a big bag of little scripts
  if [[ $batch = "LOADLEVELER" ]] ; then
    typeset jobstepscript=$jobscript
  else
    # submit this from $jobscript for LSF
    typeset jobstepscript="${jobscript}.${step_name}"
    # use bjgroup command to look at groups
    # use bgdel command to remove a group
    typeset jobgroup="/JEFS/ENS/${ENS_JGEValidDate}/${$}"
  fi

  # append to the LoadLeveler/LSF script
  bsub_kopt=""
  if [[ $batch = "LSF" ]] ; then
    typeset bsub_kopt=""
    if [[ $interactive_run = "TRUE" ]] ; then
      if [[ $jmejobtype = "ENSPOST" ]] ; then
        bsub_kopt="-K"
      fi
    fi
# use this hack to force interactive submission of every job step
#    bsub_kopt="-K"
  fi
  if (( step_num == 0 )) ; then
    if [[ $batch = "LSF" ||  $batch = "NO_QUEUE" ]] ; then
      # for LSF, $jobscript is a ksh script
      print "#!/bin/ksh" >> $jobscript
    fi
    print "#" >> $jobscript
    print "# ${batch} JOB COMMAND FILE FOR ENS FORECAST STARTING AT ${ENS_JGEValidDate}" >> $jobscript
    if [[ $batch = "LSF" ||  $batch = "NO_QUEUE" ]] ; then
      # for LSF, $jobscript needs path
      SetPath
      print "#" >> $jobscript
      print "# explicitly load all shared functions" >> $jobscript
      print ". ${scriptdir}/ENS_SharedFunctions.ksh" >> $jobscript
      print "# explicitly set path" >> $jobscript
      print "SetPath" >> $jobscript
# TODO:  did addition of #!/bin/ksh to child scripts fix this?  
      # Should be able to call "bsub" without full path once path is 
      # set via SetPath as in commented-out code above.  But it doesn't 
      # work.  The following wacky hack does work.  
      if [[ $batch = "LSF" ]] ; then
        bsubcmd=$( which bsub )
      else
        bsubcmd=""
      fi
      print "bsubcmd=${bsubcmd}" >> $jobscript
      jobscript_out="${jobscript}.out.${$}"
      print "jobscript_out=${jobscript_out}" >> $jobscript
      print "ensemble_dir=${ensemble_dir}" >> $jobscript
      print "print \"Using path = \\\"\${PATH}\\\"\" >> \${jobscript_out}" >> $jobscript
      print "print \"Using bsubcmd = \\\"\${bsubcmd}\\\"\" >> \${jobscript_out}" >> $jobscript
    fi
  fi
  print "#" >> $jobscript
  if [[ $batch = "LSF" || $batch = "NO_QUEUE" ]] ; then
    # for LSF, submit steps from top-level job script
    # use "bsub -K" to make jobs submission interactive
    # TODO need to clean up after dead jobs anymore?
    jobssbase=$(/usr/bin/basename ${jobstepscript})
    if [[ $batch = "LSF" ]] ; then
      first="  ( \${bsubcmd} ${bsub_kopt} < \${ensemble_dir}/${jobssbase} )"
      last=" >> \${jobscript_out} 2>&1"
    else
      first="  ( \${ensemble_dir}/${jobssbase} )"
      last=" >> \${jobscript_out} 2>&1"
    fi
    print "  print \"${first}\"${last}" >> $jobscript
    print "  ${first} ${last}" >> $jobscript
    # also, need this in child scripts
    print "#!/bin/ksh" >> $jobstepscript
    print "#" >> $jobstepscript
  fi
  execbase=$(/usr/bin/basename $executable)
  if [[ $jmejobtype = "ENSPOST" ]] ; then
    print "# JOB STEP FOR ${execbase}" >> $jobstepscript
  elif [[ $jmejobtype = "ENSFAIL" ]] ; then
    print "# JOB STEP FOR FAILURE PRIOR TO ENSPOST" >> $jobstepscript
  else
    print "# JOB STEP FOR ${execbase} FOR ENSEMBLE MEMBER ${thisensemble}" >> $jobstepscript
  fi
  print "#" >> $jobstepscript
  if [[ $batch = "LOADLEVELER" ]] ; then
    # only specify job_name once
    if (( step_num == 0 )) ; then
      print "#@ job_name            = ENS_${ENS_JGEValidDate}" >> $jobstepscript
      print "#" >> $jobstepscript
    fi
  else
    print "#BSUB -J ${step_name}" >> $jobstepscript
  fi
  # specify dependencies on other jobs steps
  # each step is dependent upon the previous step and possibly on even earlier 
  # steps, except for WRFWPS steps which may start immediately
  WRF_preproc="WPS"
  if [[ $jmejobtype != "WRF${WRF_preproc}" ]] ; then
    if [[ $jmejobtype = "WRFREAL_${WRF_preproc}" ]] ; then
      # A WRFREAL_WPS step depends only on successful completion 
      # of the previous WRFWPS step for this ensemble member.  
      typeset prev_step_name="WRF${WRF_preproc}_${thisensemble}"
      if [[ $batch = "LSF" ]] ; then
        prev_step_name="WRF${WRF_preproc}_${thisensemble}.${$}"
      fi
      if [[ $batch = "LOADLEVELER" ]] ; then
        print "#@ dependency          = ${prev_step_name} == 0" >> $jobstepscript
      else
        print "#BSUB -w ended(\"${prev_step_name}\")" >> $jobstepscript
      fi
    elif [[ $jmejobtype = "WRFMODEL" ]] ; then
      # A WRFMODEL step depends on successful completion of the previous 
      # WRFREAL_WPS step for this ensemble member.  
      # When using LoadLeveler it also depends upon WRFREAL_WPS steps for all 
      # ensemble members having already been queued (to avoid idle CPUs 
      # during model runs due one or more members getting too far behind).  
      # Since LoadLeveler does not provide dependencies on job steps being 
      # queued, we use the next best thing which is successful completion of 
      # the job before the previous job.  Thus, when using LoadLeveler, a 
      # WRFMODEL step will not start until WRFWPS steps have successfully 
      # completed for all ensemble members.  (Note that this is an 
      # optimzation designed to keep the AFWA machine as busy as possible 
      # during the ENS run window.  It is not dependent upon batch queueing 
      # system.)  
      typeset prev_step_name="WRFREAL_${WRF_preproc}_${thisensemble}"
      if [[ $batch = "LSF" ]] ; then
        prev_step_name="WRFREAL_${WRF_preproc}_${thisensemble}.${$}"
      fi
      typeset otherdeps=""
      for anensemble in ${memberId[*]} ; do
        otherdeps="${otherdeps} && (WRF${WRF_preproc}_${anensemble} == 0)"
      done
      if [[ $batch = "LOADLEVELER" ]] ; then
        print "#@ dependency          = (${prev_step_name} == 0) ${otherdeps}" >> $jobstepscript
      else
        print "#BSUB -w ended(\"${prev_step_name}\")" >> $jobstepscript
      fi
    elif [[ $jmejobtype = "WRFPOST" ]] ; then
      # A WRFPOST step depends only upon successful completion of the previous 
      # WRFMODEL step even at AFWA because WRFPOST is run in a different 
      # queue and there are no following job steps that use big chunks of the 
      # machine.  Thanks to Steve Rugg for this suggestion.  
      typeset prev_step_name="WRFMODEL_${thisensemble}"
      if [[ $batch = "LSF" ]] ; then
        prev_step_name="WRFMODEL_${thisensemble}.${$}"
      fi
      if [[ $batch = "LOADLEVELER" ]] ; then
        print "#@ dependency          = (${prev_step_name} == 0)" >> $jobstepscript
      else
        print "#BSUB -w ended(\"${prev_step_name}\")" >> $jobstepscript
      fi
    elif [[ $jmejobtype = "ENSPOST" ]] ; then
      # The ENSPOST step depends on successful completion of the previous 
      # WRFPOST steps for every ensemble member.  
      typeset prev_step_name="WRFPOST_${memberId[0]}"
      if [[ $batch = "LSF" ]] ; then
        prev_step_name="WRFPOST_${memberId[0]}.${$}"
      fi
      typeset otherdeps=""
      anensemble=1
      until (( anensemble >= maxensemble )) ; do
        if [[ $batch = "LOADLEVELER" ]] ; then
          otherdeps="${otherdeps} && (WRFPOST_${memberId[anensemble]} == 0)"
        else
          otherdeps="${otherdeps} && ended(\"WRFPOST_${memberId[anensemble]}.${$}\")"
        fi
        (( anensemble += 1 ))
      done
      if [[ $batch = "LOADLEVELER" ]] ; then
        print "#@ dependency          = (${prev_step_name} == 0)${otherdeps}" >> $jobstepscript
      else
        print "#BSUB -w 'ended(\"${prev_step_name}\") ${otherdeps}'" >> $jobstepscript
      fi
      if [[ $batch = "LSF" ]] ; then
        # for LSF, submit steps from top-level job script
        # for LSF, print $jobgroup from $jobscript (takes the place of 
        # LoadLeveler $jobid)
        # TODO:  This would fail if the last call to AddJobStep was not 
        # TODO:  for "ENSPOST".  Fix this if needed.  
        print "  print \"${jobgroup}\"" >> $jobscript
      fi
    elif [[ $jmejobtype = "ENSFAIL" ]] ; then
      # The ENSFAIL step depends on ENSPOST not being run due to some error(s) 
      # in previous job step(s).  
      typeset prev_step_name="ENSPOST"
      if [[ $batch = "LOADLEVELER" ]] ; then
        print "#@ dependency          = (${prev_step_name} == CC_NOTRUN)" >> $jobstepscript
      fi
    fi
  fi

#print "DEBUG:  step_name = <${step_name}>"
#print "DEBUG:  node_usage = <${node_usage}>"
#print "DEBUG:  job_name = <${job_name}>"

if [[ $batch = "LOADLEVELER" ]] ; then
  cat >> $jobstepscript << EOF1
#@ step_name           = ${step_name}
#@ class               = ${class}
#@ job_type            = ${job_type}
EOF1
else
  print "#BSUB -q ${class}" >> $jobstepscript
fi

  if [[ $job_type = "parallel" ]] ; then
    if [[ $batch = "LOADLEVELER" ]] ; then
      cat >> $jobstepscript << EOF2
#@ node                = ${node}
#@ tasks_per_node      = ${ENS_CONFIG_TasksPerNode}
#@ network.MPI         = csss,shared,us
EOF2
    else
      typeset -i ntasks
      ntasks=${node}*${ENS_CONFIG_TasksPerNode}
      print "#BSUB -n ${ntasks}" >> $jobstepscript
      print "#BSUB -R \"span[ptile=${ENS_CONFIG_TasksPerNode}]\"" >> $jobstepscript
    fi
  else
    if [[ $batch = "LSF" ]] ; then
      print "#BSUB -n 1" >> $jobstepscript
    fi
  fi

  if [[ $wall_clock_limit != "unlimited" ]] ; then
    if [[ $batch = "LOADLEVELER" ]] ; then
      print "#@ wall_clock_limit    = ${wall_clock_limit}:00" >> $jobstepscript
    else
      print "#BSUB -W ${wall_clock_limit}" >> $jobstepscript
    fi
  fi

if [[ $batch = "LOADLEVELER" ]] ; then
  cat >> $jobstepscript << EOF3
#@ environment         = COPY_ALL;MP_EUILIB=us
#@ notification        = never
#@ node_usage          = ${node_usage}
#@ checkpoint          = no
#@ executable          = ${executable}
#@ arguments           = ${scriptdir} ${thisensemble} ${ensemble_dir} ${*} ${logfile} ${email_target}
#@ initialdir          = ${initdir}
#@ output              = ${step_name}.\$(jobid).\$(stepid).out
#@ error               = ${step_name}.\$(jobid).\$(stepid).err
#@ queue
EOF3
else
  cat >> $jobstepscript << EOF4
#BSUB -P ${ENS_CONFIG_Account}
#BSUB -o ${initdir}/${step_name}.out
#BSUB -e ${initdir}/${step_name}.err
#BSUB -g ${jobgroup}
print "path = \"\${PATH}\""
cd ${initdir}
${executable} ${scriptdir} ${thisensemble} ${ensemble_dir} ${*} ${logfile} ${email_target} ${config_tag}
EOF4

chmod a+x ${jobstepscript} || \
  ErrorEmailExit "failed to chmod job script ${jobstepscript}" \
    $self $logfile $email_target ""
fi

# $$$ TODO:  Add a dependence on ENSPOST not being run to set up "FAIL" flag 
# $$$        files.  
# if [[ $jmejobtype = "ENSPOST" ]] ; then
#      print "#@ dependency          = ENSPOST == CC_NOTRUN" >> $jobstepscript
#      print "..." >> $jobstepscript
#      print "#@ queue"  >> $jobstepscript
# fi

}


# Find the most recent valid data sets that can be used as input to 
# ENS.  This script is AFWA-specific.  Dates are returned in variables 
# ENS_JGEValidDate, ENS_agrmetValidDate, ENS_sstValidDate
# Arguments are:  
#   logfile        Name of logfile for writing error messages.  
#   email_address  Email address to which error messages may be sent (NCAR 
#                  only).  
# Usage:
#   GetMostRecentValidDate logfile email_address
# Return values:
#   ENS_JGEValidDate is set to YYYYMMDDHH of most recent JGE data set.  
#   ENS_agrmetValidDate is set to YYYYMMDDHH of corresponding agrmet data set.
#   ENS_sstValidDate is set to YYYYMMDDHH of corresponding sst data set.  
#
# Dependencies: 
#  GetMyConfig must have already been called by the caller so ensemble-wide 
#  ENS_CONFIG_* variables are valid.  
#
function GetMostRecentValidDate {
  typeset logfile=$1
  typeset email_address=$2

  # set up paths to data
  typeset JGE_data=${ENS_CONFIG_JGEDataPath}
  typeset agrmet_data=${ENS_CONFIG_agrmetDataPath}
  typeset navysst_data=${ENS_CONFIG_navysstDataPath}

#print "DEBUG GetMostRecentValidDate:  JGE_data = <${JGE_data}>"
#print "DEBUG GetMostRecentValidDate:  agrmet_data = <${agrmet_data}>"
#print "DEBUG GetMostRecentValidDate:  navysst_data = <${navysst_data}>"

  #
  # Find the most recent ready JGE dataset(s)...  
  #
  set -A filedates "false" "false" "false" "false"
  set -A enshours 00 06 12 18
  typeset indx=0
  for enshour in ${enshours[*]} ; do
    typeset readyfile="${JGE_data}/${enshour}Z/ensemble_ready"
    if [[ -f ${readyfile} ]] ; then
      filedates[$indx]=$(cat ${readyfile})${enshour}
#print "DEBUG new filedate = <${filedates[$indx]}>"
    fi
    (( indx += 1 ))
  done

  # pick out dates of existing ensemble_ready files and write to 
  # tmp file for sorting
  typeset tmpfile=tmp_dates.$$
  \rm -f $tmpfile
  for filedate in ${filedates[*]} ; do
    if [[ $filedate != "false" && $filedate != "" ]] ; then
      print $filedate >> $tmpfile
    fi
  done
#print "DEBUG tmpfile = $(cat $tmpfile)"

  # sort dates of existing ensemble_ready files so most recent is first
  set -A jgedates $( cat ${tmpfile} | sort -r )
  \rm -f $tmpfile
#print "DEBUG jgedates = <${jgedates[*]}>"

  # Of the above dates, find the most recent that also has valid agrmet and 
  # navy sst data.  
  ENS_JGEValidDate="false"
  for jgedate in ${jgedates[*]} ; do
#print "DEBUG checking agrmet and sst for jgedate = <${jgedate}> ..."
    typeset foundjge="false"
    typeset foundagrmet="false"
    typeset foundsst="false"
    typeset enshour=$( echo $jgedate | cut -c 9-10 )
    typeset ensembleready=${JGE_data}/${enshour}Z/ensemble_ready
    # make sure that the ensemble_ready file has not been removed
    if [[ -f $ensembleready ]] ; then
#print "DEBUG:  FOUND JGE for jgedate = <${jgedate}> ..."
      foundjge="true"
      agrmetfile=${agrmet_data}/${jgedate}/agrmet.grib.03hr.${jgedate}
      if [[ -f $agrmetfile ]] ; then
#print "DEBUG:  FOUND AGRMET for jgedate = <${jgedate}> ..."
        foundagrmet="true"
#      else  # DEBUG
#print "DEBUG:  DID NOT FIND AGRMET for jgedate = <${jgedate}> ..."
      fi
      navysstfile=${navysst_data}/${jgedate}/navyssts
      if [[ -f $navysstfile ]] ; then
        ensyear=$( echo $jgedate | cut -c 1-4 )
        sstline=$( ${ENS_CONFIG_g1print} ${navysstfile} | grep ${ensyear} | grep -v Attempting )
        if (( $? == 0 )) ; then
#print "DEBUG:  FOUND SST for jgedate = <${jgedate}> ..."
          foundsst="true"
#        else  # DEBUG
#print "DEBUG:  DID NOT FIND SST for jgedate = <${jgedate}> ..."
        fi
      fi
#    else  # DEBUG
#print "DEBUG:  DID NOT FIND JGE for jgedate = <${jgedate}> ..."
    fi
    if [[ $foundjge = "true" && \
          $foundagrmet = "true" && \
          $foundsst = "true" ]] ; then
      ENS_JGEValidDate=$jgedate
      ENS_agrmetValidDate=$jgedate
#print "DEBUG:  Eureka, ENS_JGEValidDate = <${ENS_JGEValidDate}> ..."
      break
    fi
#print "DEBUG done checking agrmet and sst for jgedate = <${jgedate}> ..."
#print "DEBUG foundjge = <${foundjge}> ..."
#print "DEBUG foundagrmet = <${foundagrmet}> ..."
#print "DEBUG foundsst = <${foundsst}> ..."
  done

  # return date or exit on error
  if [[ $ENS_JGEValidDate = "false" ]] ; then
    ErrorEmailExit "Failed to find a valid recent data set" \
      "GetMostRecentValidDate" $logfile $email_address ""
  else
# parse out NAVYSST valid time too...
    typeset ensyear=$( echo $ENS_JGEValidDate | cut -c 1-4 )
    typeset sstlinea=${ensyear}${sstline#*${ensyear}}
    typeset navysstdate=${sstlinea%:*}
    typeset sst_year=$( echo $navysstdate | cut -c 1-4 )
    typeset sst_month=$( echo $navysstdate | cut -c 6-7 )
    typeset sst_day=$( echo $navysstdate | cut -c 9-10 )
    typeset sst_hour=$( echo $navysstdate | cut -c 12-13 )
    ENS_sstValidDate="${sst_year}${sst_month}${sst_day}${sst_hour}"
  fi

}



# Force the specific date for a retrospective run.  
# Dates are returned in variables $ENS_JGEValidDate, $ENS_agrmetValidDate, 
# and $ENS_sstValidDate .  
# Arguments are:  
#   logfile        Name of logfile for writing error messages.  
#   email_address  Email address to which error messages may be sent (NCAR 
#                  only).  
#   ValidDate      Valid date for GFS
# Usage:
#   ForceValidDate logfile email_address ValidDate
# Return values:
#   ENS_JGEValidDate is set to YYYYMMDDHH of JGE data set.  
#   ENS_agrmetValidDate is set to YYYYMMDDHH of corresponding agrmet data set.  
#   ENS_sstValidDate is set to YYYYMMDDHH of corresponding sst data set.  
#
function ForceValidDate {
  typeset logfile=$1
  typeset email_address=$2
  typeset ValidDate=$3

  # set up paths to data
  typeset JGE_data=${ENS_CONFIG_JGERetroDataPath}
  typeset agrmet_data=${ENS_CONFIG_agrmetDataPath}
  typeset navysst_data=${ENS_CONFIG_navysstDataPath}

#print "DEBUG ForceValidDate:  JGE_data = <${JGE_data}>"
#print "DEBUG ForceValidDate:  agrmet_data = <${agrmet_data}>"
#print "DEBUG ForceValidDate:  navysst_data = <${navysst_data}>"

  typeset jgedate=${ValidDate}
  # With the JGE date, assign AGRMET and SST files.
  agrmetfile=${agrmet_data}/${jgedate}/agrmet.grib.03hr.${jgedate}
  if [[ ! -f $agrmetfile ]] ; then
    ErrorEmailExit "Failed to find an agrmet data set valid at ${jgedate}" \
      "ForceValidDate" $logfile $email_address $jgedate
  fi
  navysstfile=${navysst_data}/${jgedate}/navyssts
  if [[ ! -f $navysstfile ]] ; then
    ErrorEmailExit "Failed to find a Navy SST data set valid at ${jgedate}" \
      "ForceValidDate" $logfile $email_address $jgedate
  fi
  ensyear=$( echo $jgedate | cut -c 1-4 )
  sstline=$( ${ENS_CONFIG_g1print} ${navysstfile} | grep ${ensyear} | grep -v Attempting | grep -v File )
  ENS_JGEValidDate=$jgedate
  ENS_agrmetValidDate=$jgedate

# parse out NAVYSST valid time too...
  typeset ensyear=$( echo $ENS_JGEValidDate | cut -c 1-4 )
  typeset sstlinea=${ensyear}${sstline#*${ensyear}}
  typeset navysstdate=${sstlinea%:*}
  typeset sst_year=$( echo $navysstdate | cut -c 1-4 )
  typeset sst_month=$( echo $navysstdate | cut -c 6-7 )
  typeset sst_day=$( echo $navysstdate | cut -c 9-10 )
  typeset sst_hour=$( echo $navysstdate | cut -c 12-13 )
  ENS_sstValidDate="${sst_year}${sst_month}${sst_day}${sst_hour}"

}


function Usage {
  print "ENS_EnsembleDriver.ksh - ENS ensemble driver"
  print
  print "Usage:"
  print
  print "  ENS_EnsembleDriver.ksh [-i] [-m] ensemble_top [retrospective_date]"
  print
  print "  -i                  interactive:  wait until all jobs are done before returning"
  print "  -m                  do MSS writes instead of post (NCAR only)"
  print "  ensemble_top        full path to top-level ensemble directory"
  print "  retrospective_date  [optional] YYYYMMDDHH for retrospective run"
}


# Check for standard "SUCCESS" messages in rsl.out.0000 file.  
function CheckRSLOutput {
  typeset runpath=$1; shift
  typeset self=$1; shift
  typeset exec_path=$1; shift
  typeset thisensemble=$1; shift
  typeset scriptdir=$1; shift
  typeset logfile=$1; shift
  typeset email_target=$1; shift
  typeset valid_date=$1; shift

  typeset stdOutFile=${runpath}/rsl.out.0000
  typeset exec_name=$(/usr/bin/basename $exec_path)
  # Since we cannot rely upon error status return values in LSF, we must check 
  # files on disk.  
  # Since we cannot rely upon files written by one node being immediately visible to 
  # all nodes in file systems like GPFS/NFS/..., we'll sleep if the file is not 
  # visible right away.  Yuck!  
  typeset i=1
  typeset numtries=2
  until (( i > numtries )) ; do
  if [[ -f $stdOutFile ]] ; then
     tail ${stdOutFile} | grep "SUCCESS COMPLETE" > /dev/null 2>&1
     if (( $? == 0 )) ; then
        AppendToLogFile "${self}:  ${exec_name} SUCCESS for ensemble member ${thisensemble}" \
          $self $scriptdir $logfile $email_target
     else
        ErrorEmailExit "real.exe FAILURE for ensemble member ${thisensemble}" \
           $self $logfile $email_target $valid_date
     fi
     break
  else
     if (( i < numtries )) ; then
       # LSF hackery
       sleep 10
     else
       typeset errMessage="The file ${stdOutFile} does not exist."
       ErrorEmailExit "$errMessage" $self $logfile $email_target $valid_date
     fi
  fi
  (( i += 1 ))
  done
}

# Check for standard "SUCCESS" messages in an arbitrary file
function CheckRegularLogs {
  typeset runpath=$1; shift
  typeset self=$1; shift
  typeset exec_path=$1; shift
  typeset thisensemble=$1; shift
  typeset scriptdir=$1; shift
  typeset logfile=$1; shift
  typeset email_target=$1; shift
  typeset valid_date=$1; shift
  typeset filetocheck=$1; shift

  typeset stdOutFile=${runpath}/${filetocheck}
  typeset exec_name=$(/usr/bin/basename $exec_path)
  typeset i=1
  typeset numtries=2
  until (( i > numtries )) ; do
  if [[ -f $stdOutFile ]] ; then
     tail ${stdOutFile} | grep "SUCCESS COMPLETE" > /dev/null 2>&1
     if (( $? == 0 )) ; then
        AppendToLogFile "${self}:  ${exec_name} SUCCESS for ensemble member ${thisensemble}" \
          $self $scriptdir $logfile $email_target
     else
        ErrorEmailExit "real.exe FAILURE for ensemble member ${thisensemble}" \
           $self $logfile $email_target $valid_date
     fi
     break
  else
     if (( i < numtries )) ; then
       # LSF hackery
       sleep 10
     else
       typeset errMessage="The file ${stdOutFile} does not exist."
       ErrorEmailExit "$errMessage" $self $logfile $email_target $valid_date
     fi
  fi
  (( i += 1 ))
  done
}

