#!/bin/ksh
#==========================================================================
#
# Script Name:  ENS_WRFVar.ksh
#
# TODO:  Get scientist to look at namelist
#
# Description:  
#
#   Calls WRF-Var for a given ensemble member.
#
# Usage:  
#
#   ENS_WRFVar.ksh $scriptdir $thisensemble $ensemble_dir \
#                   $ENS_ValidDate $logfile $email_target
#
# Example: ENS_WRFVar.ksh \ 
#          /home/bluevista/schramm/JEFS_2.2/ENS/ENS_scripts 1 \
#          /ptmp/schramm/ENS_Ensembles/2007042900  2007042900 \
#          /ptmp/schramm/ENS_Ensembles/2007042900/wrfvar.log schramm@ucar.edu
#Should be  /ptmp/schramm/ENS_Ensembles/logs/YYYYMMDD_timestamp/ENS_WRFVar.log
#
# Input data files:       
#     ${OB_DIR}/${DATE}/obs.ascii           # Output from OBSPROC
#     ${BE_DIR}/gen_be.NMC.dat              # Background error file
#     ${RC_DIR}/$DATE/wrfinput_d${DOMAIN}   # If cold start
#     ${RC_DIR}/$DATE/wrfbdy_d$DOMAIN}      # If cold start
#     ${FC_DIR}/$DATE/wrfbdy_d$DOMAIN       # If cycling and warm start and this |
#     ${FC_DIR}/${PREV_DATE}/wrf_3dvar_input_d${DOMAIN}_${ANALYSIS_DATE} # <-----|
#
# Output data files:
#     ${FC_DIR}/${PREV_DATE}/analysis
#
#     $scriptdir           absolute path to these scripts
#     $thisensemble        member number
#     $ensemble_dir        top-level working directory (includes date dir)
#     $ENS_ValidDate       valid start date of forecast
#     $logfile             full path to logfile
#     $email_target        where to send email if something fails
#
# Author:  
#
#   Julie Schramm                          email:  schramm@ucar.edu
#     NCAR, MMM Division                   voice:  303 497 8191
#     3450 Mitchell Lane                     fax:  303 497 8181
#     Boulder, Colorado 80301-2260 USA
#
#==========================================================================

# name of this script
self=$(/usr/bin/basename $0)

#-------------------------------------------------------------------------
# load arguments
#-------------------------------------------------------------------------
scriptdir=$1; shift
thisensemble=$1; shift
ensemble_dir=$1; shift
ENS_ValidDate=$1; shift
logfile=$1; shift
email_target=$1; shift

#-------------------------------------------------------------------------
# explicitly load all shared functions and set path
#-------------------------------------------------------------------------
. ${scriptdir}/ENS_SharedFunctions.ksh
SetPath

AppendToLogFile "${self}:  BEGIN WRF-Var for ${ENS_ValidDate} at $(date -u)" \
                           $self $scriptdir $logfile $email_target

#-------------------------------------------------------------------------
# parse ensemble-wide and member-specific config files
# GetMyConfig sets variables named ENS_CONFIG_*
#-------------------------------------------------------------------------
GetMyConfig $thisensemble $scriptdir $logfile $email_target
runpath="${ensemble_dir}/${ENS_CONFIG_RunPath}"
GetBatch  # sets ENS_Batch

#----------------------------------------------------------------------
# Set environment variables from CONFIG vars
#----------------------------------------------------------------------
#export CLEAN=$ENS_CONFIG_CLEAN               # Remove work dirs if true
#export EXPT=$ENS_CONFIG_EXPT_NAME            # Experiment name
#export DATE=$ENS_ValidDate                   # Analysis date
#export FCST_RANGE=$ENS_CONFIG_FCST_RANGE     # Cycle period, not used in obsproc
#export REL_DIR=$ENS_CONFIG_DATC_TOP          # Top level for DATC code
#export WRFVAR_DIR=$ENS_CONFIG_WRFVAR_TOP     # Top level for WRF-Var code

export DEBUG=true
export CLEAN=false                          # Remove work dirs if true
export RUN_CMD="mpirun.lsf"                 # MPI jobs for bluevista
export FIRST=true                           # true if no data in /fc
export EXPT="ENS"                           # Experiment name
export FCST_RANGE=6                         # Cycle period, not used in obsproc
export NUM_PROCS=8                          # Number of processors
export NL_TIME_STEP=360                     # Shared with real, wrf

export DATE=$ENS_ValidDate                  # Current date
export WINDOW_START=0
export WINDOW_END=0
export NL_NUM_FGAT1
export CYCLING=false
export CYCLE_PERIOD=6
export DOMAIN=01

#Default directories/files:

export REL_DIR=/home/bluevista/schramm/code # Top level for DATC code
export WRF_DIR=$REL_DIR/WRFV2
export WRFVAR_DIR=$REL_DIR/wrfvar           # Top level for WRF-Var code
export WRFPLUS_DIR=$REL_DIR/wrfplus

export DAT_DIR=$runpath/wrfvar              # Top level for wrfvar output
export EXP_DIR=$DAT_DIR                     # Top dir for particular exp. 
export RUN_DIR=$DAT_DIR                     # Run dir for experiment 
export WORK_DIR=$RUN_DIR/working            # Work directory , used by CLEAN 
export OB_DIR=$ensemble_dir/obsproc/data    # Top level obs dir for ob.ascii files
                                            # ${DATE} added by DATC scrip
export BE_DIR=/ptmp/schramm/data/con200/be  # Background error file
export RC_DIR=/ptmp/schramm/data/con200/rc
export FC_DIR=/ptmp/schramm/data/con200/da_cycling/fc
export BIASCORR_DIR=/home/bluevista/schramm/code/wrfvar/run/biascorr

#DALE: Boundaries look wrong to me.
export DA_BOUNDARIES=/ptmp/schramm/data/con200/rc/2007042900/wrfbdy_d01

export DA_ANALYSIS=/ptmp/schramm/data/rtobs
# wrfvar background errors.
export DA_BACK_ERRORS=/mmm/users/wrfhelp/data/con200/be/gen_be.NMC.dat

#export RTTOV=/homebv/wrhelp/external/rttov/rttov85                            # RTTOV
#export DA_RTTOV_COEFFS=$RTTOV/rtcoef_rttov7
#export CRTM=/homebv/wrfhelp/external/crtm
#export DA_CRTM_COEFFS=$CRTM/crtm_coefs

export NL_GLOBAL=${NL_GLOBAL:-false}
export NL_VAR4D=${NL_VAR4D:-false}
export NL_RUN_HOURS=${NL_RUN_HOURS:-6}
export NL_JCDFI_USE=${NL_JCDFI_USE:-false}

echo " Getting svn version numbers"
#export WRFVAR_VN=`svnversion -n \$WRFVAR_DIR 2>/dev/null`
#export WRF_VN=`svnversion -n \$WRF_DIR 2>/dev/null`
#export WRFPLUS_VN=`svnversion -n \$WRFPLUS_DIR 2>/dev/null`

if $DEBUG; then
   echo "WRFVAR_VN        $WRFVAR_VN"
   echo "WRF_VN           $WRF_VN"
   echo "WRFPLUS_VN       $WRFPLUS_VN"
fi

ERR='<FONT COLOR="red">'
END='</FONT>'

if $DEBUG; then
   echo "DA_FIRST_GUESS        $DA_FIRST_GUESS"
   echo "DA_BOUNDARIES         $DA_BOUNDARIES"
   echo "DA_BACK_ERRORS        $DA_BACK_ERRORS"
fi

#----------------------------------------------------------------------
# These namelist variables were added to get the same namelist as
# that used in the conus 200 case.  The NL_* variables exported from
# this file are used by $WRFVAR_DIR/build/inc/namelist_script.inc
#----------------------------------------------------------------------
export NL_WRITE_INCREMENTS=.true.
export NL_OB_FORMAT=2
export NL_HISTORY_INTERVAL=180
export NL_FRAMES_PER_OUTFILE=1
export NL_DEBUG_LEVEL=0
export NL_INPUTOUT_INTERVAL=360
export NL_INPUTOUT_BEGIN_H=6
export NL_INPUT_OUTNAME="wrf_3dvar_input_d<domain>_<date>"
export NL_WRITE_INPUT=.true.

export NL_E_WE=45
export NL_E_SN=45
export NL_E_VERT=28
export NL_NUM_METGRID_LEVELS=27
export NL_P_TOP_REQUESTED=5000
export NL_DX=200000
export NL_DY=200000
export NL_SMOOTH_OPTION=1
export NL_ETA_LEVELS=" 1.000, 0.990, 0.978, 0.964, 0.946, 0.922, 0.894, 0.860, 0.817, 0.766, 0.707, 0.644, 0.576, 0.507, 0.444, 0.380, 0.324, 0.273, 0.228, 0.188, 0.152, 0.121, 0.093, 0.069, 0.048, 0.029, 0.014, 0.000"
export NL_MP_PHYSICS=3
export NL_RA_LW_PHYSICS=1
export NL_RA_SW_PHYSICS=1
export NL_RADT=30
export NL_SF_SFCLAY_PHYSICS=1
export NL_SF_SURFACE_PHYSICS=1
export NL_BL_PBL_PHYSICS=1
export NL_CU_PHYSICS=1
export NL_CUDT=5
export NL_NUM_SOIL_LAYERS=5
export NL_MP_ZERO_OUT=2

export NL_W_DAMPING=0
export NL_DIFF_OPT=0
export NL_KM_OPT=1 
export NL_DAMPCOEF=0.2
export NL_TIME_STEP_SOUND=6
export NL_BASE_TEMP=290.0
export NL_JCDFI_USE=false
export NL_SPECIFIED=.true.

echo " Done setting the variables for member ${thisensemble}" 
#---------------------------------------------------------------------- 
# Set the directories needed from CONFIG variables 
#---------------------------------------------------------------------- 
mkdir -p $DAT_DIR || ErrorEmailExit "could not create WRF-Var run directory \ 
                                    for ensemble member ${ENS_CONFIG_MemberName}" \ 
                                    $self $logfile $email_target $ENS_ValidDate
#----------------------------------------------------------------------
# Check for existence of input data file
#----------------------------------------------------------------------
obs_file=${OB_DIR}/${DATE}/ob.ascii
echo " Set obs_file to ${obs_file}"

if [[ ! -e $obs_file ]]; then
       errMessage="The input file ${obs_file} for WRF-Var does not exist."
       ErrorEmailExit "$errMessage" $self $logfile $email_target $ENS_ValidDate
fi

YEAR=`echo $DATE | cut -c1-4`
MONTH=`echo $DATE | cut -c5-6`
DAY=`echo $DATE | cut -c7-8`
HOUR=`echo $DATE | cut -c9-10`
export ANALYSIS_DATE=${YEAR}-${MONTH}-${DAY}_${HOUR}:00:00

#----------------------------------------------------------------------
# Run DATC da_run_wrfvar.ksh script
#----------------------------------------------------------------------

export DA_FIRST_GUESS=${RC_DIR}/$DATE/wrfinput_d${DOMAIN}
if $CYCLING; then
   if ! $FIRST; then
      export DA_FIRST_GUESS=${FC_DIR}/${PREV_DATE}/wrf_3dvar_input_d${DOMAIN}_${ANALYSIS_DATE}
   fi
fi
export DA_ANALYSIS=$FC_DIR/$DATE/analysis

$WRFVAR_DIR/scripts/da_trace.ksh da_run_wrfvar $RUN_DIR
${WRFVAR_DIR}/scripts/da_run_wrfvar.ksh > $RUN_DIR/index.html 2>&1


RC=$?
if test $RC != 0; then
   echo `date` "${ERR}Failed with error $RC$END"
   exit 1
fi
export WRF_INPUT=$DA_ANALYSIS

#----------------------------------------------------------------------
# Do lots of error checking after, since there is minimal in black box
#----------------------------------------------------------------------
wrfVar_status=$?
if (( $wrfVar_status != 0 )) ; then
  ErrorEmailExit "da_run_wrfvar.ksh failed, see log file ${logfile}" \
    $self $logfile $email_target $ENS_ValidDate
fi

AppendToLogFile "${self}: END LOG FOR ENS_WRFVar.ksh for member ${thisensemble}" \
                 $self $scriptdir $logfile $email_target

#----------------------------------------------------------------------
# Need a "SUCCESS" message in output files to determine failure
#----------------------------------------------------------------------

AppendToLogFile "${self}: END WRF-Var for member ${thisensemble} at $(date -u)" \
                 $self $scriptdir $logfile $email_target

#----------------------------------------------------------------------
# Begin cleanup
#----------------------------------------------------------------------
if $CLEAN; then rm -rf $WORK_DIR; fi

#----------------------------------------------------------------------
# Function to unset export variables
#----------------------------------------------------------------------
unset CLEAN
unset FIRST
unset EXPT
unset DATE
unset MAX_OB_RANGE
unset FCST_RANGE
unset NUM_PROCS
unset NL_TIME_STEP
unset WINDOW_START
unset WINDOW_END
unset NL_NUM_FGAT1
unset CYCLING
unset CYCLE_PERIOD
unset DOMAIN

unset REL_DIR
unset WRF_DIR
unset WRFVAR_DIR
unset WRFPLUS_DIR
unset DAT_DIR
unset REG_DIR
unset EXP_DIR
unset RUN_DIR
unset WORK_DIR
unset OB_DIR
unset BE_DIR
unset RC_DIR
unset FC_DIR
unset BIASCORR_DIR
unset DA_BOUNDARIES
unset DA_ANALYSIS
unset DA_BACK_ERRORS
unset RTTOV
unset DA_RTTOV_COEFFS
unset CRTM
unset DA_CRTM_COEFFS
unset NL_GLOBAL
unset NL_VAR4D
unset NL_RUN_HOURS
unset NL_JCDFI_USE
unset WRFVAR_VN
unset WRF_VN
unset WRFPLUS_VN
unset ERR
unset END

unset NL_WRITE_INCREMENTS
unset NL_OB_FORMAT
unset NL_HISTORY_INTERVAL
unset NL_FRAMES_PER_OUTFILE
unset NL_DEBUG_LEVEL
unset NL_INPUTOUT_INTERVAL
unset NL_INPUTOUT_BEGIN_H
unset NL_INPUT_OUTNAME
unset NL_WRITE_INPUT

unset NL_E_WE
unset NL_E_SN
unset NL_E_VERT
unset NL_NUM_METGRID_LEVELS
unset NL_P_TOP_REQUESTED
unset NL_DX
unset NL_DY
unset NL_SMOOTH_OPTION
unset NL_ETA_LEVELS
unset NL_MP_PHYSICS
unset NL_RA_LW_PHYSICS
unset NL_RA_SW_PHYSICS
unset NL_RADT
unset NL_SF_SFCLAY_PHYSICS
unset NL_SF_SURFACE_PHYSICS
unset NL_BL_PBL_PHYSICS
unset NL_CU_PHYSICS
unset NL_CUDT
unset NL_NUM_SOIL_LAYERS
unset NL_MP_ZERO_OUT

unset NL_W_DAMPING
unset NL_DIFF_OPT
unset NL_KM_OPT
unset NL_DAMPCOEF
unset NL_TIME_STEP_SOUND
unset NL_BASE_TEMP
unset NL_JCDFI_USE

exit 0

