#!/bin/ksh
#
# Script Name:
#
#   ENS_WRF_Model.ksh
#
# Usage:  
#
#   ENS_WRF_Model.ksh $scriptdir $thisensemble $ensemble_dir \
#                     $ENS_ValidDate $logfile $email_target
#
#     $scriptdir     absolute path to these scripts
#     $thisensemble  ensemble number
#     $ensemble_dir  top-level working directory
#     $ENS_ValidDate valid start date of forecast
#     $logfile       full path to logfile
#     $email_target  where to send email if something fails
#     $config_tag    posterior tag on configs and namelists, e.g. DEMO
#
# Description:  
#
#   Runs WRF wrf.exe and checks for job completion.
#
# Dependencies:  
#
#   Since this script is intended to be called from a script called from cron, 
#   it has no dependencies on environment variables.  
#
#   Scripts called by this script must reside in the same directory as 
#   this script.  
#
# Author:  
#
#   Tom Henderson                          email:  hender@ucar.edu
#   NCAR, MMM Division                     voice:  303 497 1628
#   3450 Mitchell Lane                       fax:  303 497 8181
#   Boulder, Colorado 80301-2260 USA
#

set -u

# name of this script
self=$(/usr/bin/basename $0)

# load arguments
scriptdir=$1; shift
thisensemble=$1; shift
ensemble_dir=$1; shift
ENS_ValidDate=$1; shift
logfile=$1; shift
email_target=$1; shift
config_tag=$1; shift

# explicitly load all shared functions
. ${scriptdir}/ENS_SharedFunctions.ksh
# explicitly set path
SetPath

AppendToLogFile "${self}:  WRF MODEL BEGIN for member ${thisensemble} at $(date -u)" \
  $self $scriptdir $logfile $email_target

ncl_exe=`which ncl`

# parse config file for $thisensemble
# GetMyConfig sets variables named ENS_CONFIG_*
GetMyConfig $thisensemble $scriptdir $logfile $email_target $config_tag

set -A pArr `echo $ENS_CONFIG_RandomVariance | sed s/,/\ /g`
set -A vArr `echo $ENS_CONFIG_RandomVars | sed s/,/\ /g`

# how to error check here?
if [[ ! -e $ENS_CONFIG_PertScript ]] ; then
   ErrorEmailExit "Could not find perturbation script $ENS_CONFIG_PertScript" \
   ${self} $logfile $email_target ${ENS_JGEValidDate}
fi

# loop thru domains then variables
thisdomain=1
until (( thisdomain > ENS_CONFIG_NumDomains )) ; do
  wrfdomain="d0${thisdomain}"
  export DOMAIN=$thisdomain
  export MEMBER=$thisensemble
  /bin/ln -sf wrfinput_$wrfdomain wrfinput_${wrfdomain}.nc
  
  vcnt=0
  for vName in ${vArr[*]}; do
    export PVARIABLE=$vName
    export PVARIANCE=${pArr[$vcnt]}
    export VNUM=$vcnt
    $ncl_exe < $ENS_CONFIG_PertScript #> $logfile
    (( vcnt += 1 ))
  done 

  /bin/rm -f wrfinput_${wrfdomain}.nc
  (( thisdomain += 1 ))
done

# turn on for debugging
if [[ $ENS_CONFIG_DEBUG = "TRUE" ]] ; then
  set -x
fi

runpath="${ensemble_dir}/${ENS_CONFIG_RunPath}"
# point to executable
wrf_exec=${ENS_CONFIG_ExecPath}/wrf.exe
if [[ ! -x  ${wrf_exec} ]] ; then
  ErrorEmailExit "${wrf_exec} is not executable" \
    $self $logfile $email_target $ENS_ValidDate
fi

#----------------------------------------------------------------------
# look for "SUCCESS" message in real.exe output files and exit on failure
# Note that this is redundant with ENS_WRF_Real_WPS.ksh because LSF may 
# cause scripts to die in the middle!!!  
#----------------------------------------------------------------------
real_exec=${ENS_CONFIG_RealExecPath}/real.exe

GetMPILauncher  # sets $ENS_MPILauncher
if [[ $ENS_MPILauncher ]] ; then
  CheckRSLOutput ${runpath} ${self} ${real_exec} ${thisensemble} \
                 ${scriptdir} ${logfile} ${email_target} $ENS_ValidDate
else
  CheckRegularLogs ${runpath} ${self} ${real_exec} ${thisensemble} \
                 ${scriptdir} ${logfile} ${email_target} $ENS_ValidDate \
                 real.log     
fi

# stow results from real.exe away so wrf won't overwrite rsl.*.* files, etc.
stowdir=${runpath}/real.out.$$
mkdir ${stowdir} || \
  ErrorEmailExit "Could not create directory ${stowdir}" \
    $self $logfile $email_target $ENS_ValidDate
if [[ -f rsl.out.0000 ]] ; then
  mv ${runpath}/rsl.*.* ${stowdir} || \
    ErrorEmailExit "Failed to stow rsl files in ${stowdir}" \
      $self $logfile $email_target $ENS_ValidDate
else
  mv ${runpath}/real.log ${stowdir} || \
    ErrorEmailExit "Failed to stow real.log in ${stowdir}" \
      $self $logfile $email_target $ENS_ValidDate
fi
if [[ -f *.out ]] ; then
  mv ${runpath}/*.out ${stowdir} || \
    ErrorEmailExit "Failed to stow stdout file in ${stowdir}" \
      $self $logfile $email_target $ENS_ValidDate
fi
if [[ -f *.err ]] ; then
mv ${runpath}/*.err ${stowdir} || \
  ErrorEmailExit "Failed to stow stderr file in ${stowdir}" \
    $self $logfile $email_target $ENS_ValidDate
fi
CopyFileF ${runpath}/namelist.input ${stowdir} $logfile $email_target $ENS_ValidDate
MoveWRFTables ${runpath} ${stowdir} $logfile $email_target $ENS_ValidDate

# get model tables for wrf.exe
# Note that tables used for real.exe and wrf.exe are allowed to differ.  This 
# was needed at one point.  
GetWRFTables ${ENS_CONFIG_ModelDataPath} ${runpath} $logfile $email_target $ENS_ValidDate

#----------------------------------------------------------------------
# Run the WRF model
#----------------------------------------------------------------------

AppendToLogFile "${self}:  Executing ${wrf_exec} for ensemble member ${thisensemble}" \
  $self $scriptdir $logfile $email_target
GetMPILauncher  # sets $ENS_MPILauncher
$ENS_MPILauncher $wrf_exec > wrf.log

if (( $? != 0 )) ; then
   logMessage="Problem submitting ${wrf_exec} in ${ENS_MPILauncher}, see log file ${logfile}"
   ErrorEmailExit "$logMessage" $self $logfile $email_target $ENS_ValidDate
fi

#----------------------------------------------------------------------
# look for "SUCCESS" message in output files and exit on failure
#----------------------------------------------------------------------
if [[ $ENS_MPILauncher ]] ; then
  CheckRSLOutput ${runpath} ${self} ${wrf_exec} ${thisensemble} \
                 ${scriptdir} ${logfile} ${email_target} $ENS_ValidDate
else
  CheckRegularLogs ${runpath} ${self} ${wrf_exec} ${thisensemble} \
                 ${scriptdir} ${logfile} ${email_target} $ENS_ValidDate \
                 wrf.log     
fi

AppendToLogFile "${self}:  WRF MODEL END for member ${thisensemble} at $(date -u)" \
  $self $scriptdir $logfile $email_target

exit 0

