#!/bin/ksh
#
# Script Name:
#
#   ENS_WRF_Real_WPS.ksh
#
# Usage:  
#
#   ENS_WRF_Real_WPS.ksh $scriptdir $thisensemble $ensemble_dir \
#                        $ENS_ValidDate $logfile $email_target
#
#     $scriptdir     absolute path to these scripts
#     $thisensemble  ensemble number
#     $ensemble_dir  top-level working directory
#     $ENS_ValidDate valid start date of forecast
#     $logfile       full path to logfile
#     $email_target  where to send email if something fails
#
# Description:  
#
#   Runs WRF real.exe and checks for successful completion.  
#
# Dependencies:  
#
#   Assumes that WRF WPS has completed successfully.  
#
#   Since this script is intended to be called from a script called from cron, 
#   it has no dependencies on environment variables.  
#
#   Scripts called by this script must reside in the same directory as 
#   this script.  
#
# Author:  
#
#   Tom Henderson                          email:  hender@ucar.edu
#   NCAR, MMM Division                     voice:  303 497 1628
#   3450 Mitchell Lane                       fax:  303 497 8181
#   Boulder, Colorado 80301-2260 USA
#
#   Modifications by Josh Hacker (JPH Aug 29 2006) for WPS version of real.exe

set -u

# name of this script
self=$(/usr/bin/basename $0)

# load arguments
scriptdir=$1; shift
thisensemble=$1; shift
ensemble_dir=$1; shift
ENS_ValidDate=$1; shift
logfile=$1; shift
email_target=$1; shift
config_tag=$1; shift

# explicitly load all shared functions
. ${scriptdir}/ENS_SharedFunctions.ksh
# explicitly set path
SetPath

AppendToLogFile "${self}:  WRF REAL BEGIN for member ${thisensemble} at $(date -u)" \
  $self $scriptdir $logfile $email_target

# parse config file for $thisensemble
# GetMyConfig sets variables named ENS_CONFIG_*
GetMyConfig $thisensemble $scriptdir $logfile $email_target $config_tag
runpath="${ensemble_dir}/${ENS_CONFIG_RunPath}"
GetBatch  # sets ENS_Batch

# turn on for debugging
if [[ $ENS_CONFIG_DEBUG = "TRUE" ]] ; then
  set -x
fi

# check for metgrid log files
ls -1 ${runpath}/metgrid.log* > /dev/null 2>&1
if (( $? != 0 )) ; then 
  if [[ $ENS_Batch = "LSF" ]] ; then
    # attempt a resubmission one-time-only for LSF
    # TBH:  Note that this hideous hack is required to work around the 
    # TBH:  LSF issues that still will not go away...  
    # TBH:  This method is expensive since "bsub -K" will cause this step 
    # TBH:  to stall until the WPS step runs.  This can be improved but it 
    # TBH:  ultimately requires generation of new JCF steps with new pids to 
    # TBH:  make dependencies work correctly.  Ugh.  
    resub_flag_file="${runpath}/WPS_already_rerun"
    if [[ -f $resub_flag_file ]] ; then
      ErrorEmailExit "WPS failed to write metgrid log files after one resubmission" \
        $self $logfile $email_target $ENS_ValidDate
    else
      touch $resub_flag_file
      AppendToLogFile "${self}:  Attempting to resubmit WPS job..." \
        $self $scriptdir $logfile $email_target
      # first clean up...  
      \rm -f met_em.d* FILE* ungrib.log GRIBFILE* namelist.* Vtable AGR geo_em.d* NavySST > /dev/null 2>&1
      # now resubmit
      jcf=$( ls -1 ${ensemble_dir}/WRF_ENS.lsf.WRFWPS_${thisensemble}* ) || \
        ErrorEmailExit "could not find WPS JCF" \
          $self $logfile $email_target $ENS_ValidDate
      AppendToLogFile "${self}:  Resubmitting WPS JCF ${jcf}..." \
        $self $scriptdir $logfile $email_target
      bsub_ret=$( bsub -K < $jcf ) || \
        ErrorEmailExit "failed to submit JCF ${jcf}" \
          $self $logfile $email_target $ENS_ValidDate
      AppendToLogFile "${self}:  bsub returned ${bsub_ret}..." \
        $self $scriptdir $logfile $email_target
    fi
  else
    ErrorEmailExit "WPS failed to write metgrid log files" \
      $self $logfile $email_target $ENS_ValidDate
  fi
fi

# set up run directory for real.exe
# for now, copy all files since we do not know if they will be on the 
# same file systems (if not, "ln -s" wouldn't work)
AppendToLogFile "${self}:  Setting up run directory for real.exe for ensemble ${thisensemble}" \
  $self $scriptdir $logfile $email_target
# point to executable
real_exec=${ENS_CONFIG_RealExecPath}/real.exe
if [[ ! -x  ${real_exec} ]] ; then
  ErrorEmailExit "${real_exec} is not executable" \
    $self $logfile $email_target $ENS_ValidDate
fi

# get LANDUSE.TBL
if [[ $ENS_CONFIG_LANDUSELocation = "ENS_CONFIG_ModelDataPath" ]] ; then
  LANDUSEPath="$ENS_CONFIG_ModelDataPath"
elif [[ $ENS_CONFIG_LANDUSELocation = "scriptdir" ]] ; then
  LANDUSEPath="$scriptdir/../ENS_LANDUSE"
else
  ErrorEmailExit "invalid value for \$ENS_CONFIG_LANDUSELocation" \
    $self $logfile $email_target $ENS_ValidDate
fi
GetFile "${LANDUSEPath}/${ENS_CONFIG_LANDUSEFile}" "${runpath}/LANDUSE.TBL" $logfile $email_target $ENS_ValidDate

# get model tables for real.exe
GetWRFTables ${ENS_CONFIG_RealDataPath} ${runpath} $logfile $email_target $ENS_ValidDate

#----------------------------------------------------------------------
# Build "real" input data files for WRF from WPS data sets
#----------------------------------------------------------------------

AppendToLogFile "${self}:  Executing ${real_exec} for ensemble member ${thisensemble}" \
  $self $scriptdir $logfile $email_target
GetMPILauncher  # sets $ENS_MPILauncher
$ENS_MPILauncher $real_exec > real.log

stat=$?
if (( $stat != 0 )) ; then
   logMessage="WARNING:  ${ENS_MPILauncher} returned ${stat} in ${real_exec}, log file is ${logfile}"
   AppendToLogFile "$logMessage" ${self} $scriptdir $logfile $email_target
fi

#----------------------------------------------------------------------
# look for "SUCCESS" message in output files and exit on failure
#----------------------------------------------------------------------
GetMPILauncher  # sets $ENS_MPILauncher
if [[ $ENS_MPILauncher ]] ; then
  CheckRSLOutput ${runpath} ${self} ${real_exec} ${thisensemble} \
                 ${scriptdir} ${logfile} ${email_target} $ENS_ValidDate
else
  CheckRegularLogs ${runpath} ${self} ${real_exec} ${thisensemble} \
                 ${scriptdir} ${logfile} ${email_target} $ENS_ValidDate \
                 real.log
fi

AppendToLogFile "${self}:  WRF REAL END for member ${thisensemble} at $(date -u)" \
  $self $scriptdir $logfile $email_target

exit 0

