#!/bin/ksh
#
# Script Name:
#
#   ENS_WRF_WPS.ksh
#
# Usage:  
#
#   ENS_WRF_WPS.ksh $scriptdir $thisensemble $ensemble_dir \
#                  $ENS_JGEValidDate $ENS_agrmetValidDate \
#                  $ENS_sstValidDate $RetrospectiveRun $logfile $email_target
#
#     $scriptdir           absolute path to these scripts
#     $thisensemble        ensemble number
#     $ensemble_dir        top-level working directory
#     $ENS_JGEValidDate    valid date of most recent complete data set
#     $ENS_agrmetValidDate valid date of corresponding agrment data set
#     $ENS_sstValidDate    valid date of corresponding sst data set
#     $RetrospectiveRun    TRUE if this is a retrospective run, FALSE for a 
#                          standard production run.  
#     $logfile             full path to logfile
#     $email_target        where to send email if something fails
#
# Description:  
#
#   Runs WRF WPS, checks for successful completion of WPS, and creates 
#   namelists for real.exe and wrf.exe.  
#
# Dependencies:  
#
#   Since this script is intended to be called from a script called from cron, 
#   it has no dependencies on environment variables.  
#
#   Scripts called by this script must reside in the same directory as 
#   this script.  
#
# Author:  
#
#   Tom Henderson                          email:  hender@ucar.edu
#   NCAR, MMM Division                     voice:  303 497 1628
#   3450 Mitchell Lane                       fax:  303 497 8181
#   Boulder, Colorado 80301-2260 USA
#
# Adaptation of ENS_WRF_SI.ksh to WPS version of WRF by Josh Hacker (JPH)

set -u

# name of this script
self=$(/usr/bin/basename $0)

# load arguments
scriptdir=$1; shift
thisensemble=$1; shift
ensemble_dir=$1; shift
ENS_JGEValidDate=$1; shift
ENS_agrmetValidDate=$1; shift
ENS_sstValidDate=$1; shift
RetrospectiveRun=$1; shift
logfile=$1; shift
email_target=$1; shift
config_tag=$1; shift

# explicitly load all shared functions
. ${scriptdir}/ENS_SharedFunctions.ksh
# explicitly set path
SetPath

AppendToLogFile "${self}:  WRF WPS BEGIN for member ${thisensemble} at $(date -u)" \
  $self $scriptdir $logfile $email_target

# parse config file for $thisensemble
# GetMyConfig sets variables named ENS_CONFIG_*
GetMyConfig $thisensemble $scriptdir $logfile $email_target $config_tag

# turn on for debugging
if [[ $ENS_CONFIG_DEBUG = "TRUE" ]] ; then
  set -x
fi

runpath="${ensemble_dir}/${ENS_CONFIG_RunPath}"
GetSite  # sets ENS_Site

AppendToLogFile "${self}:  ENS_JGEValidDate = <${ENS_JGEValidDate}>" \
  $self $scriptdir $logfile $email_target
AppendToLogFile "${self}:  ENS_agrmetValidDate = <${ENS_agrmetValidDate}>" \
  $self $scriptdir $logfile $email_target
AppendToLogFile "${self}:  ENS_sstValidDate = <${ENS_sstValidDate}>" \
  $self $scriptdir $logfile $email_target
AppendToLogFile "${self}:  ENS_CONFIG_FcstLengthHrs = <${ENS_CONFIG_FcstLengthHrs}>" \
  $self $scriptdir $logfile $email_target

# run WPS and build namelist
# first, select JGE path depending on whether this is a standard production 
# run or a retrospective run
if [[ $RetrospectiveRun = "FALSE" ]] ; then
  # standard production run
  JGEDataPath=${ENS_CONFIG_JGEDataPath}
else
  # retrospective run
  start_date_no_hour=`echo $ENS_JGEValidDate | cut -c 1-8`
  JGEDataPath=${ENS_CONFIG_JGERetroDataPath}/${start_date_no_hour}
fi

set -A indices 0 1 2
set -A data_set_names SST AGRMET GFS
set -A valid_dates ${ENS_sstValidDate} ${ENS_agrmetValidDate} ${ENS_JGEValidDate} 
set -A data_paths ${ENS_CONFIG_navysstDataPath} ${ENS_CONFIG_agrmetDataPath} ${JGEDataPath}
set -A vtables ${ENS_CONFIG_WPSVtableSST} ${ENS_CONFIG_WPSVtableAGRMET} ${ENS_CONFIG_WPSVtableGFS}

# loop over all data sets
for indx in ${indices[*]} ; do

# some logging...

AppendToLogFile "${self}:  ${scriptdir}/ENS_run_wps.ksh ${scriptdir} \
  ${ENS_CONFIG_NamelistPath} ${ENS_CONFIG_Namelist} ${thisensemble} ${runpath} ${ENS_Site} ${ENS_CONFIG_WPSRoot} ${data_set_names[$indx]} \
  ${valid_dates[$indx]} ${ENS_JGEValidDate} ${ENS_CONFIG_FcstLengthHrs} ${data_paths[$indx]} ${ENS_CONFIG_JGEMember} \
  ${ENS_CONFIG_WPSGeoPath} ${vtables[$indx]} \
  ${ENS_CONFIG_FGIntervalHrs} ${ENS_CONFIG_FGNumLevels} ${logfile} \
  ${email_target}" $self $scriptdir $logfile $email_target $config_tag

AppendToLogFile "${self}:BEGIN LOG FOR ENS_run_wps.ksh ENSEMBLE MEMBER ${thisensemble} dataset ${data_set_names[$indx]}" \
  $self $scriptdir $logfile $email_target

${scriptdir}/ENS_run_wps.ksh ${scriptdir} ${ENS_CONFIG_NamelistPath} \
  ${ENS_CONFIG_Namelist} ${thisensemble} \
  ${runpath} ${ENS_Site} ${ENS_CONFIG_WPSRoot} ${data_set_names[$indx]} ${valid_dates[$indx]} \
  ${ENS_JGEValidDate} ${ENS_CONFIG_FcstLengthHrs} ${data_paths[$indx]} ${ENS_CONFIG_JGEMember} \
  ${ENS_CONFIG_WPSGeoPath} ${vtables[$indx]} \
  ${ENS_CONFIG_FGIntervalHrs} ${ENS_CONFIG_FGNumLevels} ${ENS_CONFIG_DEBUG} ${logfile} \
  ${email_target} ${config_tag}

run_wps_status=$?
if (( $run_wps_status != 0 )) ; then
  ErrorEmailExit "run_wps failed, see log file ${logfile}" \
    $self $logfile $email_target $ENS_JGEValidDate
fi

AppendToLogFile "${self}: END LOG FOR ENS_run_wps.ksh ENSEMBLE MEMBER ${thisensemble} dataset ${data_set_names[$indx]}" \
  $self $scriptdir $logfile $email_target

done

AppendToLogFile "${self}:  WRF WPS END for member ${thisensemble} at $(date -u)" \
  $self $scriptdir $logfile $email_target

exit 0

