#!/bin/ksh
#
# Script Name:
#
#   ENS_run_wps.ksh
#
# Usage:  
#
#   ENS_run_wps.ksh $scriptdir $namelistdir $wrf_nml $thisensemble $runpath \
#                   $site $WPS_root $data_set_name $ValidDate \
#                   $JGEValidDate $ENS_FcstLengthHrs \
#                   $DataPath $jge_member $ENS_WPSGeoPath $ENS_WPSVtable \
#                   $ENS_FGIntervalHrs $ENS_FGNumLevels $ENS_DEBUG \
#                   $logfile $email_target
#
# NOTE:  
#
#   GFS data set must be processed LAST.  See argument $data_set_name.  
#
# Author:
#   Josh Hacker, based on run_si.csh script (Dave Gill)
#-----------------------------------------------------------------------------

set -u

# name of this script
self=$(/usr/bin/basename $0)

NUM_REQ_ARGS=21
numArgs=$#

if [[ $numArgs -eq $NUM_REQ_ARGS ]];then   # load arguments
   scriptdir=$1; shift
   namelistdir=$1; shift
   wrf_nml=$1; shift
   thisensemble=$1; shift
   runpath=$1; shift
   site=$1; shift              # location of run ("AFWA" or "NCAR")
   WPS_root=$1; shift
   data_set_name=$1; shift     # name for data set ("SST", "AGRMET", or "GFS")
   ValidDate=$1; shift         # valid date for data set
   JGEValidDate=$1; shift      # valid date for JGE data set (needed for SST)
   ENS_FcstLengthHrs=$1; shift
   DataPath=$1; shift          # path to data set
   jge_member=$1; shift        # two-digit integer ID of JGE member
   WPSGeoPath=$1; shift
   ENS_WPSVtable=$1; shift
   ENS_FGIntervalHrs=$1; shift
   ENS_FGNumLevels=$1; shift
   ENS_DEBUG=$1; shift
   logfile=$1; shift
   email_target=$1; shift
   config_tag=$1; shift
else
   ErrorExit "requires ${NUM_REQ_ARGS} arguments, you provided ${numArgs}" $self 
fi

if [[ $ENS_DEBUG = "TRUE" ]] ; then
  set -x
fi

# explicitly load all shared functions
. ${scriptdir}/ENS_SharedFunctions.ksh

# explicitly set path
SetPath

cd ${runpath} || ErrorEmailExit "Could not cd to ${runpath}" \
    $self $logfile $email_target $JGEValidDate

#----------------------------------------------------------------------
# parse start date, fcst start date = $ValidDate (data start date)
#----------------------------------------------------------------------
start_year4=`echo $ValidDate | cut -c 1-4`
start_month=`echo $ValidDate | cut -c 5-6`
start_day=`echo $ValidDate | cut -c 7-8`
start_hour=`echo $ValidDate | cut -c 9-10`

#----------------------------------------------------------------------
# Check that forecast is starting at a data time interval
#----------------------------------------------------------------------
if (( start_hour % ENS_FGIntervalHrs != 0 )) ; then
   errMessage="Forecast must start at a data time: ${start_hour} ${ENS_FGIntervalHrs}"
   ErrorEmailExit "${errMessage}" $self $logfile $email_target $JGEValidDate
fi

#----------------------------------------------------------------------
# Modify data enddate so that it is equal to the next largest multiple 
# of ENS_FGIntervalHrs for WRF and WPS namelists
#         enddate >= fcst start date + ENS_FcstLengthHrs
#----------------------------------------------------------------------
#if [[ $data_set_name = "GFS" ]] ; then
  if (( ENS_FcstLengthHrs % ENS_FGIntervalHrs == 0 )) ; then
     (( flen_ss = ENS_FcstLengthHrs * 3600 ))
     enddate=`$scriptdir/increment_date.pl -s ${ValidDate}0000 -f $flen_ss`  
  else
     (( dataLengthHrs = ENS_FcstLengthHrs + (ENS_FGIntervalHrs - \
                       (ENS_FcstLengthHrs %  ENS_FGIntervalHrs)) ))
     (( dataLengthSec = dataLengthHrs * 3600 ))
     enddate=`$scriptdir/increment_date.pl -s ${ValidDate}0000 -f $dataLengthSec`  
  fi
#fi

#----------------------------------------------------------------------
# parse end date 
#----------------------------------------------------------------------
end_year4=`echo $enddate | cut -c 1-4`
end_month=`echo $enddate | cut -c 5-6`
end_day=`echo $enddate | cut -c 7-8`
end_hour=`echo $enddate | cut -c 9-10`

#----------------------------------------------------------------------
# link data files, geo files, and Vtable locally
#----------------------------------------------------------------------
logMessage="${self}:  Linking files ${WPSGeoPath}/geo_em\* ${DataPath}/\*"
AppendToLogFile "${logMessage}" ${self} $scriptdir $logfile $email_target

if [[ -e geo_em.d01.nc ]] ; then /bin/rm -f geo_em.*  ; fi

ln -sf ${WPSGeoPath}/geo_em* ./ || \
  ErrorEmailExit "Could not link ${WPSGeoPath}/geo_em\*" \
    $self $logfile $email_target $JGEValidDate

if [[ -e GRIBFILE.AAA ]] ; then /bin/rm -f GRIBFILE*  ; fi

if [[ $data_set_name = "GFS" ]] ; then
   # must filter long argument list at AFWA, might as well do it at NCAR too
   gfspath="${DataPath}/${start_hour}Z"
   logMessage="${self}:  $WPS_root/link_grib.csh ${gfspath}/MT.\*membrp${jge_member}\*" 
   AppendToLogFile "${logMessage}" ${self} $scriptdir $logfile $email_target

   $WPS_root/link_grib.csh ${gfspath}/MT.*membrp${jge_member}* || \
      ErrorEmailExit "link_grib.csh failed for JGE data" \
        $self $logfile $email_target $JGEValidDate
elif [[ $data_set_name = "AGRMET" ]] ; then
   logMessage="${self}:  $WPS_root/link_grib.csh ${DataPath}/${ValidDate}/"
   AppendToLogFile "${logMessage}" ${self} $scriptdir $logfile $email_target

   $WPS_root/link_grib.csh ${DataPath}/${ValidDate}/ || \
      ErrorEmailExit "link_grib.csh failed for AGRMET data" \
        $self $logfile $email_target $JGEValidDate
elif [[ $data_set_name = "SST" ]] ; then
   # SST requires JGEValidData to create path because it lags
   logMessage="${self}:  $WPS_root/link_grib.csh ${DataPath}/${ValidDate}/" 
   AppendToLogFile "${logMessage}" ${self} $scriptdir $logfile $email_target

   $WPS_root/link_grib.csh ${DataPath}/${JGEValidDate}/ || \
      ErrorEmailExit "link_grib.csh failed for SST data" \
        $self $logfile $email_target $JGEValidDate
else
   errMessage="ERROR, unknown data set \"${data_set_name}\"" 
   ErrorEmailExit "${errMessage}" \
     $self $logfile $email_target $JGEValidDate
fi

if [[ -e Vtable ]] ; then /bin/rm -f Vtable  ; fi
ln -sf ${ENS_WPSVtable} Vtable || ErrorEmailExit "Could not link ${ENS_WPSVtable}" \
    $self $logfile $email_target $JGEValidDate
/bin/rm -f FILE:*

#----------------------------------------------------------------------
# process namelist
# Data-set-specific namelist settings
#----------------------------------------------------------------------
if [[ $data_set_name = "SST" || $data_set_name = "AGRMET" ]] ; then
   end_year4=$start_year4
   end_month=$start_month
   end_day=$start_day
   end_hour=$start_hour
fi

logMessage="${self}:  start_date = ${start_year4}${start_month}${start_day}${start_hour}"
AppendToLogFile "${logMessage}" ${self} $scriptdir $logfile $email_target
logMessage="${self}:  end_date = ${end_year4}${end_month}${end_day}${end_hour}" 
AppendToLogFile "${logMessage}" ${self} $scriptdir $logfile $email_target

#----------------------------------------------------------------------
# convert ENS_FGIntervalHrs to seconds for WPS namelist
#----------------------------------------------------------------------
((ENS_FGIntervalSec = ENS_FGIntervalHrs * 3600 ))

nml=${namelistdir}/namelist.wps.ENS_$thisensemble
logMessage="${self}: Using WPS namelist template $nml for data set $data_set_name" 
AppendToLogFile "${logMessage}" ${self} $scriptdir $logfile $email_target

new_nml=`${scriptdir}/build_namelist_wps.ksh $scriptdir $thisensemble $nml $runpath \
   $start_year4 $start_month $start_day $start_hour \
   $end_year4 $end_month $end_day $end_hour $ENS_FGIntervalSec \
   $ENS_FcstLengthHrs $ENS_FGNumLevels \
   $logfile $email_target $config_tag`

if (( $? != 0 )) ; then
   errMessage="build_namelist_wps.ksh failed to build WPS namelist, see log file ${logfile}"
   ErrorEmailExit "${errMessage}" $self $logfile $email_target $JGEValidDate
fi

AppendToLogFile "${self}:  new_nml = ${new_nml}" ${self} $scriptdir $logfile $email_target

#----------------------------------------------------------------------
# Save a copy of this WPS namelist for diagnostics since it may get 
# overwritten by another call to this function.  
#----------------------------------------------------------------------
CopyFileF $new_nml ${new_nml}.${data_set_name} $logfile $email_target $JGEValidDate

#----------------------------------------------------------------------
# to be consistent with run_si.csh, also set up the WRF namelist when
# setting up the GFS namelist
#----------------------------------------------------------------------

if [[ $data_set_name = "GFS" ]] ; then
   nml=${namelistdir}/${wrf_nml}
   logMessage="${self}: Using WRF namelist template $nml" 
   AppendToLogFile "${logMessage}" ${self} $scriptdir $logfile $email_target
   new_nml=`${scriptdir}/build_namelist_wps.ksh $scriptdir $thisensemble $nml $runpath \
      $start_year4 $start_month $start_day $start_hour \
      $end_year4 $end_month $end_day $end_hour $ENS_FGIntervalSec \
      $ENS_FcstLengthHrs $ENS_FGNumLevels \
      $logfile $email_target $config_tag`
   if (( $? != 0 )) ; then
      errMessage="build_namelist_wps.ksh failed to build WRF namelist, see log file ${logfile}"
      ErrorEmailExit "${errMessage}" \
        $self $logfile $email_target $JGEValidDate
   fi
fi

#----------------------------------------------------------------------
# geogrid should have been run off-line, so run ungrib and metgrid 
#----------------------------------------------------------------------
GetMPILauncher                # Sets $ENS_MPILauncher

#----------------------------------------------------------------------
# run ungrib
#----------------------------------------------------------------------
logMessage="${self}:  Executing ungrib.exe for ensemble member ${thisensemble}" 
AppendToLogFile "${logMessage}" $self $scriptdir $logfile $email_target

$WPS_root/ungrib.exe

#----------------------------------------------------------------------
# look for "Successful completion" message in ungrib.log file
#----------------------------------------------------------------------
ungribFile=${runpath}/ungrib.log
if [[ -f $ungribFile ]] ; then
   tail $ungribFile | grep "Successful completion" > /dev/null 2>&1

   if (( $? == 0 )) ; then
      logMessage="${self}:  ungrib.exe SUCCESS for ensemble member ${thisensemble}" 
      AppendToLogFile "${logMessage}" $self $scriptdir $logfile $email_target
   else
      errMessage="ungrib.exe FAILURE for ensemble member ${thisensemble}"
      ErrorEmailExit "${errMessage}" \
        $self $logfile $email_target $JGEValidDate
   fi
else
   errMessage="The file ${ungribFile} does not exist."
   ErrorEmailExit "${errMessage}" $self $logfile $email_target $JGEValidDate
fi

#----------------------------------------------------------------------
# run metgrid
#----------------------------------------------------------------------
if [[ $data_set_name = "GFS" ]] ; then
   logMessage="${self}:  Executing metgrid.exe for ensemble member ${thisensemble}" 
   AppendToLogFile "${logMessage}" $self $scriptdir $logfile $email_target

   $ENS_MPILauncher $WPS_root/metgrid.exe

#----------------------------------------------------------------------
# look for "Successful completion" message in metgrid.log file
#----------------------------------------------------------------------
   metgridFile=${runpath}/metgrid.log.0000
   if [[ ! -f $metgridFile ]] ; then
     metgridFile=${runpath}/metgrid.log
   fi
   if [[ -f $metgridFile ]] ; then
      tail $metgridFile | grep "Successful completion" > /dev/null 2>&1

      if (( $? == 0 )) ; then
         logMessage="${self}:  metgrid.exe SUCCESS for ensemble member ${thisensemble}" 
         AppendToLogFile "${logMessage}" $self $scriptdir $logfile $email_target
      else
         errMessage="metgrid.exe FAILURE for ensemble member ${thisensemble}"
         ErrorEmailExit "${errMessage}" \
           $self $logfile $email_target $JGEValidDate
      fi
   else
      errMessage="The file ${metgridFile} does not exist."
      ErrorEmailExit "${errMessage}" \
        $self $logfile $email_target $JGEValidDate
   fi
fi

if [[ $data_set_name = "SST" ]] ; then
   srcfile=`ls -1 FILE:*`
   MoveFileF $srcfile NavySST $logfile $email_target $JGEValidDate
fi

if [[ $data_set_name = "AGRMET" ]] ; then
   srcfile=`ls -1 FILE:*`
   MoveFileF $srcfile AGR $logfile $email_target $JGEValidDate
fi

logMessage="${self}:  END for member ${thisensemble} at $(date -u)"
AppendToLogFile "${logMessage}" $self $scriptdir $logfile $email_target


