#!/bin/ksh
#
# Script Name:
#
#   build_namelist.ksh
#
# Usage:  
#
#   build_namelist.ksh $scriptdir $thisensemble $runpath \
#                      $start_year4 $start_month $start_day $start_hour \
#                      $end_year4 $end_month $end_day $end_hour \
#                      $fcst_len $interval $logfile $email_target
#
#     $scriptdir           absolute path to these scripts
#     $thisensemble        ensemble number
#     $runpath             run directory for this ensemble member
#     $start_year4         start year of forecast, 4-digit
#     $start_month         start month of forecast
#     $start_day           start day of forecast
#     $start_hour          start hour of forecast
#     $end_year4           end year of forecast, 4-digit
#     $end_month           end month of forecast
#     $end_day             end day of forecast
#     $end_hour            end hour of forecast
#     $fcst_len            length of forecast
#     $interval            lbc interval
#     $logfile             full path to logfile
#     $email_target        where to send email if something fails
#
# Description:  
#
#   Creates namelists for real.exe and wrf.exe.  
#
# Notes:  
#
#   Once run_si.csh has been converted to run_si.ksh, this script should be 
#   converted to a function.  
#
# Author:  
#
#   Tom Henderson                          email:  hender@ucar.edu
#   NCAR, MMM Division                     voice:  303 497 1628
#   3450 Mitchell Lane                       fax:  303 497 8181
#   Boulder, Colorado 80301-2260 USA
#

set -u

# name of this script
self=$(/usr/bin/basename $0)

# load arguments
scriptdir=$1; shift
thisensemble=$1; shift
runpath=$1; shift
start_year4=$1; shift
start_month=$1; shift
start_day=$1; shift
start_hour=$1; shift
end_year4=$1; shift
end_month=$1; shift
end_day=$1; shift
end_hour=$1; shift
fcst_len=$1; shift
interval=$1; shift
logfile=$1; shift
email_target=$1; shift

# explicitly load all shared functions
. ${scriptdir}/ENS_SharedFunctions.ksh
# explicitly set path
SetPath

valid_date=${start_year4}${start_month}${start_day}${start_hour}

cd ${runpath} || \
  ErrorEmailExit "Could not cd to ${runpath}" \
    $self $logfile $email_target $valid_date

# parse config file for $thisensemble
# GetMyConfig sets variables named ENS_CONFIG_*
GetMyConfig $thisensemble $scriptdir $logfile $email_target

# turn on for debugging
if [[ $ENS_CONFIG_DEBUG = "TRUE" ]] ; then
  set -x
fi

(( history_interval_min = ${ENS_CONFIG_HistoryInterval} * 60 ))

AppendToLogFile "${self}:  Creating namelist for ensemble ${thisensemble} ..." \
  $self $scriptdir $logfile $email_target

# get namelist, insert correct dates (etc.), and put it in the run directory
namelist_template=${ENS_CONFIG_NamelistPath}/${ENS_CONFIG_Namelist}
namelist_tmp=${runpath}/namelist.input.template
namelist_out=${runpath}/namelist.input
if [[ -f ${namelist_tmp} ]] ; then
  \rm -f ${namelist_tmp}
fi
if [[ -f ${namelist_out} ]] ; then
  \rm -f ${namelist_out}
fi
GetFile ${namelist_template} "${namelist_tmp}" $logfile $email_target $valid_date
sed -e "s/_SYY_/$start_year4/g" \
    -e "s/_SMM_/$start_month/g" \
    -e "s/_SDD_/$start_day/g" \
    -e "s/_SHH_/$start_hour/g" \
    -e "s/_EYY_/$end_year4/g" \
    -e "s/_EMM_/$end_month/g" \
    -e "s/_EDD_/$end_day/g" \
    -e "s/_EHH_/$end_hour/g" \
    -e "s/_FCST_LEN_/$fcst_len/g" \
    -e "s/_HISTI_/$history_interval_min/g" \
    -e "s/_INTERVAL_/$interval/g" \
    -e "s/_MAX_DOM_/$ENS_CONFIG_NumDomains/g" \
    ${namelist_tmp} > ${namelist_out}

if (( $? == 0 )) ; then
  AppendToLogFile "${self}:  Done creating namelist ${namelist_out} for ensemble member ${thisensemble}" \
    $self $scriptdir $logfile $email_target
else
  ErrorEmailExit "Failed to create namelist ${namelist_out} for ensemble member ${thisensemble}" \
    $self $logfile $email_target $valid_date
fi

\rm -f ${namelist_tmp}

exit 0

