#!/usr/bin/perl
#=======================================================================
#
# SCRIPT:  getMemberIds.pl
#
# USAGE (command line): getMemberIds.pl -s $inputString
#       (from ksh    ): memberString=`./getMemberIds.pl -s ${inputString}`
#
# PURPOSE: Parse a character string of integers seperated by "," and "-"
#          to create a character of all integers seperated by spaces.
#          Integers seperated by "-" are expanded, and integers
#          separated by "," are put directly into the output string.
#          Output string is put in numerical order.
#
#   Input:  inputString:  string of integers seperated by "," and/or "-".
#
#   Output: string of ordered integers parsed from inputString.  
#
#   Examples:   Input                 Output
#               1,2,3,4               1 2 3 4
#               1                     1
#               1-4                   1 2 3 4
#               8,9,1-4               1 2 3 4 8 9
#
# AUTHORS:
#
#   Julie Schramm                          email:  schramm@ucar.edu
#     NCAR, MMM Division                   voice:  303 497 8191
#     3450 Mitchell Lane                     fax:  303 497 8181
#     Boulder, Colorado 80301-2260 USA
#
#=======================================================================
use strict;
use vars qw($opt_s);
use Getopt::Std;

my $filename = "getMemberIds.pl";
my $debug = 0;               # Set to 1 to turn on debugging

my @commaArray;              # Array of comma seperated elements
my $howMany;                 # Number of elements in @commaArray
my $eachComma;               # Each element of @commaArray
my @saveFillValues;          # Collect all filled values
my @fillArray;               # Intermediate array of filled-in member IDs
my @dashArray;               # Strings with dashes
my $nd;                      # Index of a string with dashes
my @saveAllMemberIDs;        # Collect single and filled-in member IDs
my $iDash;                   # Location of 1st dash in input string
my $iComma;                  # Location of 1st comma in input string
my @idArray;                 # Array of IDs returned to ksh
my $memberString;            # opt_s with quotes

getopts('s:');

if ($debug) {print "$filename:  Input string is = $opt_s\n\n";}

die "$filename: USAGE: $filename -s memberString \n" if ( ! $opt_s );

$memberString = "$opt_s";
if ($debug) {print "$filename: memberString = $memberString\n";}

#-----------------------------------------------------------------------
# Error check input string
#-----------------------------------------------------------------------
$iComma = index($memberString,",");   # Returns -1 if not found
$iDash = index($memberString,"-");

if ($debug) {print "$filename: Commas at: $iComma\n";}
if ($debug) {print "$filename: Dashes at: $iDash\n";}

#-----------------------------------------------------------------------
# Split string by commas
#-----------------------------------------------------------------------
@commaArray = split /,/, $memberString;
$howMany = scalar(@commaArray);
if ($debug) {print "$filename: How Many = $howMany\n";}
if ($debug) {print "$filename: Result: @commaArray\n";}

foreach $eachComma (@commaArray) {     # Loop through comma seperated input

    if ($debug) {print "$filename: Looping through: $eachComma $iDash\n";}

   #--------------------------------------------------------------------
   # Search each element for dashes
   #--------------------------------------------------------------------
   $iDash = index($eachComma,"-");
   if ($debug) {print "$filename: Dashes at: $iDash\n";}

   if ($iDash != -1) {                     # Found a dash
      if ($debug) {print "$filename: I found a dash in: $eachComma\n";}

      #--------------------------------------------------------------
      # Seperate string from commaArray that has a dash
      #--------------------------------------------------------------
      @dashArray = split /-/, $eachComma;
      if ($debug) {print "$filename: Filling in between: @dashArray\n";}

      #--------------------------------------------------------------
      # Fill in between 2 numbers in @dashArray
      #--------------------------------------------------------------
      if ( scalar(@dashArray) ==2 ) {
         @fillArray = ($dashArray[0] .. $dashArray[1]);
         if ($debug) {print "$filename: Filled with @fillArray\n";}
      } else {
         die "$filename: Can't fill in between more than two numbers.
                         Check input string: $memberString.\n";
      }

      #--------------------------------------------------------------
      # Add new values to end of @saveAllMemberIDs array
      #--------------------------------------------------------------
      push(@saveAllMemberIDs, @fillArray);
      if ($debug) {print "$filename: New array is @saveAllMemberIDs\n";}

   } else {                                   
   #--------------------------------------------------------------------
   # No dashes, save the number
   #--------------------------------------------------------------------
      if ($debug) {print "$filename: I found NO dashes in: $eachComma\n";}
      push(@saveAllMemberIDs, $eachComma);
   }
}

#-----------------------------------------------------------------------
# Sort an array numerically (sort @array) sorts by ASCII order
#-----------------------------------------------------------------------

@idArray = sort { $a <=> $b } @saveAllMemberIDs;
if ($debug) {print "$filename: Sorted array is: @idArray\n";}
print "@idArray";
