#!/bin/ksh
#
# Modified for AFWA ENS post-processing by Tom Henderson.  2006/6/13
#
# Original Author:  Christopher Harrop (see below)
#
# Usage:
#
#   post.ksh $scriptdir $thisensemble $ensemble_dir \
#      $ENS_ValidDate $fcst_hr $timestr $wrf_domain \
#      $AFWA_theater $logfile $email_target
#
#     $scriptdir     absolute path to these scripts
#     $thisensemble  ensemble number
#     $ensemble_dir  top-level working directory
#     $ENS_ValidDate valid start date of forecast
#     $fcst_hr       forecast hour for file being processed
#     $timestr       timestamp in name of file being processed
#     $wrf_domain    domain being processed
#     $AFWA_theater  AFWA theater ID
#     $logfile       full path to logfile
#     $email_target  where to send email if something fails
#
#dis
#dis    Open Source License/Disclaimer, Forecast Systems Laboratory
#dis    NOAA/OAR/FSL, 325 Broadway Boulder, CO 80305
#dis
#dis    This software is distributed under the Open Source Definition,
#dis    which may be found at http://www.opensource.org/osd.html.
#dis
#dis    In particular, redistribution and use in source and binary forms,
#dis    with or without modification, are permitted provided that the
#dis    following conditions are met:
#dis
#dis    - Redistributions of source code must retain this notice, this
#dis    list of conditions and the following disclaimer.
#dis
#dis    - Redistributions in binary form must provide access to this
#dis    notice, this list of conditions and the following disclaimer, and
#dis    the underlying source code.
#dis
#dis    - All modifications to this software must be clearly documented,
#dis    and are solely the responsibility of the agent making the
#dis    modifications.
#dis
#dis    - If significant modifications or enhancements are made to this
#dis    software, the FSL Software Policy Manager
#dis    (softwaremgr@fsl.noaa.gov) should be notified.
#dis
#dis    THIS SOFTWARE AND ITS DOCUMENTATION ARE IN THE PUBLIC DOMAIN
#dis    AND ARE FURNISHED "AS IS."  THE AUTHORS, THE UNITED STATES
#dis    GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND
#dis    AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS
#dis    OF THE SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE.  THEY ASSUME
#dis    NO RESPONSIBILITY (1) FOR THE USE OF THE SOFTWARE AND
#dis    DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
#dis
#dis

##########################################################################
#
#Script Name: post.ksh
# 
#     Author: Christopher Harrop
#             Forecast Systems Laboratory
#             325 Broadway R/FST
#             Boulder, CO. 80305
#
#   Released: 10/30/2003
#    Version: 1.0
#    Changes: None
#
# Purpose: This script post processes wrf output.  It is based on scripts
#          whose authors are unknown.
#
#               EXE_ROOT = The full path of the post executable
#          MOAD_DATAROOT = Top level directory of wrf output and
#                          configuration data.
#             START_TIME = The cycle time to use for the initial time. 
#              FCST_TIME = The two-digit forecast that is to be posted
# 
# A short and simple "control" script could be written to call this script
# or to submit this  script to a batch queueing  system.  Such a "control" 
# script  could  also  be  used to  set the above environment variables as 
# appropriate  for  a  particular experiment.  Batch  queueing options can
# be  specified on the command  line or  as directives at  the top of this
# script.  A set of default batch queueing directives is provided.
#
##########################################################################

set -u

# name of this script
self=$(/usr/bin/basename $0)

# load arguments
scriptdir=$1; shift
thisensemble=$1; shift
ensemble_dir=$1; shift
ENS_ValidDate=$1; shift
fcst_hr=$1; shift
timestr=$1; shift
wrf_domain=$1; shift
AFWA_theater=$1; shift
logfile=$1; shift
email_target=$1; shift

# explicitly load all shared functions
. ${scriptdir}/ENS_SharedFunctions.ksh
# explicitly set path
SetPath

# parse config file for $thisensemble
# GetMyConfig sets variables named ENS_CONFIG_*
GetMyConfig $thisensemble $scriptdir $logfile $email_target

# turn on for debugging
if [[ $ENS_CONFIG_DEBUG = "TRUE" ]] ; then
  set -x
fi

runpath="${ensemble_dir}/${ENS_CONFIG_RunPath}"

# Make sure we are using GMT time zone for time computations
export TZ="GMT"

export WRF_ROOT=${ENS_CONFIG_ModelDataPath}
export EXE_ROOT=${ENS_CONFIG_WRFPostExecPath}
export MOAD_DATAHOME=${scriptdir}/../wrfpost
export MOAD_DATAROOT=${runpath}
export START_TIME=${ENS_ValidDate}
if (( $fcst_hr < 10 )) ; then
  export FCST_TIME="0${fcst_hr}"
elif (( $fcst_hr < 100 )) ; then
  export FCST_TIME=${fcst_hr}
else
  ${ECHO} "POST_FAIL:  ERROR: fcst_hr = <${fcst_hr}> must be less than 100"
  exit 1
fi

export MODEL="ARW"

# Set up paths to shell commands
LN=ln
RM=rm
MKDIR=mkdir
CP=cp
MV=mv
ECHO=echo
CAT=cat
GREP=grep
CUT=cut
SED=sed
DATE=date
GetMPILauncher  # sets $ENS_MPILauncher
POE=$ENS_MPILauncher

# Print run parameters
${ECHO}
${ECHO} "post.ksh started at `${DATE} -u` for member ${thisensemble}"
${ECHO}
${ECHO} "MOAD_DATAROOT = ${MOAD_DATAROOT}"
${ECHO} "     EXE_ROOT = ${EXE_ROOT}"

# Check to make sure the EXE_ROOT var was specified
if [ ! -d ${EXE_ROOT} ]; then
  ${ECHO} "POST_FAIL:  ERROR: EXE_ROOT, '${EXE_ROOT}', does not exist"
  exit 1
fi

# Set up some constants
export CORE=NCAR
export POST="${EXE_ROOT}/wrfpost.exe"

# Check to make sure the post executable exists
if [ ! -x ${POST} ]; then
  ${ECHO} "POST_FAIL:  ERROR: ${POST} does not exist, or is not executable"
  exit 1
fi

# Check to make sure that the MOAD_DATAROOT exists
if [ ! -d ${MOAD_DATAROOT} ]; then
  ${ECHO} "POST_FAIL:  ERROR: MOAD_DATAROOT, '${MOAD_DATAROOT}', does not exist"
  exit 1
fi

# If START_TIME is not defined, die
# No error check...  
if [ ! "${START_TIME}" ]; then
  ${ECHO} "POST_FAIL:  ERROR: START_TIME does not exist"
  exit 1
fi

# Print out times
${ECHO} "   START_TIME = ${START_TIME}"
${ECHO} "    FCST_TIME = ${FCST_TIME}"

# Set up the work directory and cd into it
workdir=${MOAD_DATAROOT}/postprd_${wrf_domain}/${FCST_TIME}
${RM} -rf ${workdir}
${MKDIR} -p ${workdir}
cd ${workdir}

# Set up some constants
#export XLFRTEOPTS="unit_vars=yes"
#TBH:  what is tmmark for?  
export tmmark=tm00
#export MP_SHARED_MEMORY=yes

${CAT} > itag <<EOF
${MOAD_DATAROOT}/wrfout_${wrf_domain}_${timestr}
netcdf
${timestr}
${CORE}
EOF

${CAT} > input${FCST_TIME}.prd <<EOF
${workdir}/WRFPRS.GrbF${FCST_TIME}
EOF

${RM} -f fort.*
${LN} -s ${MOAD_DATAHOME}/wrf_cntrl.parm fort.14
${LN} -s ./itag fort.19
${LN} -s ./griddef.out fort.110
${LN} -s ${WRF_ROOT}/ETAMPNEW_DATA eta_micro_lookup.dat

# Run wrfpost
${POE} ${POST}< itag
if (( $? != 0 )) ; then
   ${ECHO} "POST_FAIL:  problem submitting post.ksh in ${POE}, see log file ${logfile}"
  exit ${error}
fi

# Check to make sure all Post  output files were produced
if [ ! -s "${workdir}/WRFPRS${FCST_TIME}.tm00" ]; then
  ${ECHO} "POST_FAIL:  wrfpost crashed! WRFPRS${FCST_TIME}.tm00 is missing"
  exit 1
fi
if [ ! -s "${workdir}/WRFTWO${FCST_TIME}.tm00" ]; then
  ${ECHO} "POST_FAIL:  wrfpost crashed! WRFTWO${FCST_TIME}.tm00 is missing"
  exit 1
fi

# Create single directory in which to store post-processed results, if 
# it does not already exist
postoutdir="${ensemble_dir}/GRIB_OUT_${wrf_domain}"
if [[ ! -d ${postoutdir} ]] ; then
  mkdir -p ${postoutdir}
  if (( $? != 0 )) ; then
    ErrorEmailExit "POST_FAIL:  could not create directory for post-processed output ${postoutdir}" \
      ${self} $logfile $email_target $ENS_ValidDate
  fi
fi
# Move the output files to ${postoutdir}
griboutext="${FCST_TIME}_${wrf_domain}.${thisensemble}.tm00"
${MV} ${workdir}/WRFPRS${FCST_TIME}.tm00 ${postoutdir}/WRFPRS${griboutext}
if (( $? != 0 )) ; then
  ErrorEmailExit "POST_FAIL:  could not move WRFPRS${FCST_TIME}.tm00 to ${postoutdir}" \
    ${self} $logfile $email_target $ENS_ValidDate
fi
${MV} ${workdir}/WRFTWO${FCST_TIME}.tm00 ${postoutdir}/WRFTWO${griboutext}
if (( $? != 0 )) ; then
  ErrorEmailExit "POST_FAIL:  could not move WRFTWO${FCST_TIME}.tm00 to ${postoutdir}" \
    ${self} $logfile $email_target $ENS_ValidDate
fi

# Finally, move GRIB files to their final destination
# ultimate location for GRIB output
START_HR=$( print $START_TIME | cut -c9-10 )
postmvdir="${ENS_CONFIG_GRIBOutPath}/${START_HR}Z"
if [[ ! -d ${postmvdir} ]] ; then
  mkdir -p ${postmvdir}
  if (( $? != 0 )) ; then
    ErrorEmailExit "POST_FAIL:  could not create directory for post-processed output ${postmvdir}" \
      ${self} $logfile $email_target $ENS_ValidDate
  fi
fi
# TODO:  $$$
# TBH:  Replace the "copies" with "moves" once Steve Rugg sets up archiving 
# TBH:  of /h/data/global/GPOST/data/ensemble/ENS
#${MV} ${postoutdir}/WRFPRS${griboutext} ${postmvdir}
${CP} ${postoutdir}/WRFPRS${griboutext} ${postmvdir}
if (( $? != 0 )) ; then
  ErrorEmailExit "POST_FAIL:  could not move WRFPRS${griboutext} to ${postmvdir}" \
    ${self} $logfile $email_target $ENS_ValidDate
fi
chmod a+rw ${postmvdir}/WRFPRS${griboutext}
if (( $? != 0 )) ; then
  ErrorEmailExit "POST_FAIL:  could not chmod WRFPRS${griboutext}" \
    ${self} $logfile $email_target $ENS_ValidDate
fi
#${MV} ${postoutdir}/WRFTWO${griboutext} ${postmvdir}
${CP} ${postoutdir}/WRFTWO${griboutext} ${postmvdir}
if (( $? != 0 )) ; then
  ErrorEmailExit "POST_FAIL:  could not move WRFTWO${griboutext} to ${postmvdir}" \
    ${self} $logfile $email_target $ENS_ValidDate
fi
chmod a+rw ${postmvdir}/WRFTWO${griboutext}
if (( $? != 0 )) ; then
  ErrorEmailExit "POST_FAIL:  could not chmod WRFTWO${griboutext}" \
    ${self} $logfile $email_target $ENS_ValidDate
fi

cd ..
# TODO:  $$$
# for now, leave $workdir around for DEBUG, restore remove command later
#${RM} -rf ${workdir}

${ECHO} "post.ksh completed at `${DATE} -u` for member ${thisensemble}"

exit 0

